@@ -70,6 +70,8 @@ extern int msgb_resize_area(struct msgb *msg, uint8_t *area,
 	int old_size, int new_size);
 extern struct msgb *msgb_copy(const struct msgb *msg, const char *name);
 extern struct msgb *msgb_copy_c(const void *ctx, const struct msgb *msg, const char *name);
+extern struct msgb *msgb_copy_resize(const struct msgb *msg, uint16_t new_len, const char *name);
+extern struct msgb *msgb_copy_resize_c(const void *ctx, const struct msgb *msg, uint16_t new_len, const char *name);
 static int msgb_test_invariant(const struct msgb *msg) __attribute__((pure));
 
 /*! Free all msgbs from a queue built with msgb_enqueue().
@@ -144,6 +146,7 @@ static inline struct msgb *msgb_dequeue_count(struct llist_head *queue,
  */
 static inline unsigned int msgb_l1len(const struct msgb *msgb)
 {
+	OSMO_ASSERT(msgb->l1h);
 	return msgb->tail - (uint8_t *)msgb_l1(msgb);
 }
 
@@ -156,6 +159,7 @@ static inline unsigned int msgb_l1len(const struct msgb *msgb)
  */
 static inline unsigned int msgb_l2len(const struct msgb *msgb)
 {
+	OSMO_ASSERT(msgb->l2h);
 	return msgb->tail - (uint8_t *)msgb_l2(msgb);
 }
 
@@ -168,6 +172,7 @@ static inline unsigned int msgb_l2len(const struct msgb *msgb)
  */
 static inline unsigned int msgb_l3len(const struct msgb *msgb)
 {
+	OSMO_ASSERT(msgb->l3h);
 	return msgb->tail - (uint8_t *)msgb_l3(msgb);
 }
 
@@ -180,7 +185,8 @@ static inline unsigned int msgb_l3len(const struct msgb *msgb)
  */
 static inline unsigned int msgb_l4len(const struct msgb *msgb)
 {
-	return msgb->tail - (uint8_t *)msgb_sms(msgb);
+	OSMO_ASSERT(msgb->l4h);
+	return msgb->tail - (uint8_t *)msgb_l4(msgb);
 }
 
 /*! determine the length of the header
