@@ -322,7 +322,7 @@ static inline uint8_t *v_put(uint8_t *buf, uint8_t val)
 }
 
 /*! put (append) a TV field */
-static inline uint8_t *tv_put(uint8_t *buf, uint8_t tag, 
+static inline uint8_t *tv_put(uint8_t *buf, uint8_t tag,
 				uint8_t val)
 {
 	*buf++ = tag;
@@ -344,7 +344,7 @@ static inline uint8_t *tv_fixed_put(uint8_t *buf, uint8_t tag,
  *  \param[in] tag Tag value
  *  \param[in] val Value (in host byte order!)
  */
-static inline uint8_t *tv16_put(uint8_t *buf, uint8_t tag, 
+static inline uint8_t *tv16_put(uint8_t *buf, uint8_t tag,
 				 uint16_t val)
 {
 	*buf++ = tag;
@@ -543,7 +543,7 @@ int tlv_encode(struct msgb *msg, const struct tlv_definition *def, const struct
 int tlv_encode_ordered(struct msgb *msg, const struct tlv_definition *def, const struct tlv_parsed *tp,
 			const uint8_t *tag_order, unsigned int tag_order_len);
 
-#define TLVP_PRESENT(x, y)	((x)->lv[y].val)
+#define TLVP_PRESENT(x, y)	(!!((x)->lv[y].val))
 #define TLVP_LEN(x, y)		(x)->lv[y].len
 #define TLVP_VAL(x, y)		(x)->lv[y].val
 
