@@ -442,8 +442,8 @@ char *argv_concat(const char **argv, int argc, int shift);
 vector cmd_make_strvec(const char *);
 int cmd_make_strvec2(const char *string, char **indent, vector *strvec_p);
 void cmd_free_strvec(vector);
-vector cmd_describe_command();
-char **cmd_complete_command();
+vector cmd_describe_command(vector vline, struct vty *vty, int *status);
+char **cmd_complete_command(vector vline, struct vty *vty, int *status);
 const char *cmd_prompt(enum node_type);
 int config_from_file(struct vty *, FILE *);
 enum node_type node_parent(enum node_type);
