@@ -20,10 +20,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 #include "config.h"
@@ -35,6 +31,7 @@
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
+#include <limits.h>
 
 #include <arpa/inet.h>
 
@@ -50,9 +47,11 @@
 
 #include <osmocom/ctrl/control_cmd.h>
 #include <osmocom/ctrl/control_if.h>
+#include <osmocom/ctrl/control_vty.h>
 
 #include <osmocom/core/msgb.h>
 #include <osmocom/core/rate_ctr.h>
+#include <osmocom/core/stat_item.h>
 #include <osmocom/core/select.h>
 #include <osmocom/core/counter.h>
 #include <osmocom/core/talloc.h>
@@ -82,20 +81,16 @@ static LLIST_HEAD(ctrl_lookup_helpers);
  *  \returns 1 on success; 0 in case of error */
 int ctrl_parse_get_num(vector vline, int i, long *num)
 {
-	char *token, *tmp;
+	char *token;
+	int64_t val;
 
 	if (i >= vector_active(vline))
 		return 0;
 	token = vector_slot(vline, i);
 
-	errno = 0;
-	if (token[0] == '\0')
+	if (osmo_str_to_int64(&val, token, 10, LONG_MIN, LONG_MAX))
 		return 0;
-
-	*num = strtol(token, &tmp, 10);
-	if (tmp[0] != '\0' || errno != 0)
-		return 0;
-
+	*num = (long)val;
 	return 1;
 }
 
@@ -489,8 +484,14 @@ static int control_write_cb(struct osmo_fd *bfd, struct msgb *msg)
 		control_close_conn(ccon);
 		return -EBADF;
 	}
-	if (rc != msg->len)
+	if (rc < 0) {
 		LOGP(DLCTRL, LOGL_ERROR, "Failed to write message to the CTRL connection.\n");
+		return 0;
+	}
+	if (rc < msg->len) {
+		msgb_pull(msg, rc);
+		return -EAGAIN;
+	}
 
 	return 0;
 }
@@ -589,12 +590,12 @@ static uint64_t get_rate_ctr_value(const struct rate_ctr *ctr, int intv, const c
 	}
 }
 
-static int get_rate_ctr_group_idx(const struct rate_ctr_group *ctrg, int intv, struct ctrl_cmd *cmd)
+static int get_rate_ctr_group_idx(struct rate_ctr_group *ctrg, int intv, struct ctrl_cmd *cmd)
 {
 	unsigned int i;
 	for (i = 0; i < ctrg->desc->num_ctr; i++) {
 		ctrl_cmd_reply_printf(cmd, "%s %"PRIu64";", ctrg->desc->ctr_desc[i].name,
-				      get_rate_ctr_value(&ctrg->ctr[i], intv, ctrg->desc->group_name_prefix));
+				      get_rate_ctr_value(rate_ctr_group_get_ctr(ctrg, i), intv, ctrg->desc->group_name_prefix));
 		if (!cmd->reply) {
 			cmd->reply = "OOM";
 			return CTRL_CMD_ERROR;
@@ -723,6 +724,100 @@ static int verify_rate_ctr(struct ctrl_cmd *cmd, const char *value, void *data)
 	return 0;
 }
 
+/* Expose all stat_item groups on CTRL, as read-only variables of the form:
+ * stat_item.(last|...).group_name.N.item_name
+ * stat_item.(last|...).group_name.by_name.group_idx_name.item_name
+ */
+CTRL_CMD_DEFINE_RO(stat_item, "stat_item *");
+static int get_stat_item(struct ctrl_cmd *cmd, void *data)
+{
+	char *dup;
+	char *saveptr;
+	char *value_type;
+	char *group_name;
+	char *group_idx_str;
+	char *item_name;
+	char *tmp;
+	int32_t val;
+	struct osmo_stat_item_group *statg;
+	const struct osmo_stat_item *stat_item;
+
+	/* cmd will be freed in control_if.c after handling here, so no need to free the dup string. */
+	dup = talloc_strdup(cmd, cmd->variable);
+	if (!dup) {
+		cmd->reply = "OOM";
+		return CTRL_CMD_ERROR;
+	}
+
+	/* Split off the first "stat_item." part */
+	tmp = strtok_r(dup, ".", &saveptr);
+	if (!tmp || strcmp(tmp, "stat_item") != 0)
+		goto format_error;
+
+	/* Split off the "last." part (validated further below) */
+	value_type = strtok_r(NULL, ".", &saveptr);
+	if (!value_type)
+		goto format_error;
+
+	/* Split off the "group_name." part */
+	group_name = strtok_r(NULL, ".", &saveptr);
+	if (!group_name)
+		goto format_error;
+
+	/* Split off the "N." part */
+	group_idx_str = strtok_r(NULL, ".", &saveptr);
+	if (!group_idx_str)
+		goto format_error;
+	if (strcmp(group_idx_str, "by_name") == 0) {
+		/* The index is not given by "N" but by "by_name.foo". Get the "foo" idx-name */
+		group_idx_str = strtok_r(NULL, ".", &saveptr);
+		statg = osmo_stat_item_get_group_by_name_idxname(group_name, group_idx_str);
+	} else {
+		int idx;
+		if (osmo_str_to_int(&idx, group_idx_str, 10, 0, INT_MAX))
+			goto format_error;
+		statg = osmo_stat_item_get_group_by_name_idx(group_name, idx);
+	}
+	if (!statg) {
+		cmd->reply = "Stat group with given name and index not found";
+		return CTRL_CMD_ERROR;
+	}
+
+	/* Split off the "item_name" part */
+	item_name = strtok_r(NULL, ".", &saveptr);
+	if (!item_name)
+		goto format_error;
+	stat_item = osmo_stat_item_get_by_name(statg, item_name);
+	if (!stat_item) {
+		cmd->reply = "No such stat item found";
+		return CTRL_CMD_ERROR;
+	}
+
+	tmp = strtok_r(NULL, "", &saveptr);
+	if (tmp) {
+		cmd->reply = "Garbage after stat item name";
+		return CTRL_CMD_ERROR;
+	}
+
+	if (!strcmp(value_type, "last"))
+		val = osmo_stat_item_get_last(stat_item);
+	else
+		goto format_error;
+
+	cmd->reply = talloc_asprintf(cmd, "%"PRIu32, val);
+	if (!cmd->reply) {
+		cmd->reply = "OOM";
+		return CTRL_CMD_ERROR;
+	}
+
+	return CTRL_CMD_REPLY;
+
+format_error:
+	cmd->reply = "Stat item must be of form stat_item.type.group_name.N.item_name,"
+		" e.g. 'stat_item.last.bsc.0.msc_num:connected'";
+	return CTRL_CMD_ERROR;
+}
+
 /* counter */
 CTRL_CMD_DEFINE(counter, "counter *");
 static int get_counter(struct ctrl_cmd *cmd, void *data)
@@ -784,7 +879,7 @@ static int verify_counter(struct ctrl_cmd *cmd, const char *value, void *data)
 struct ctrl_handle *ctrl_interface_setup(void *data, uint16_t port,
 					 ctrl_cmd_lookup lookup)
 {
-	return ctrl_interface_setup_dynip(data, "127.0.0.1", port, lookup);
+	return ctrl_interface_setup2(data, port, lookup, 0);
 }
 
 static int ctrl_initialized = 0;
@@ -808,6 +903,9 @@ static int ctrl_init(unsigned int node_count)
 		goto err;
 
 	ret = ctrl_cmd_install(CTRL_NODE_ROOT, &cmd_rate_ctr);
+	if (ret)
+		goto err_vec;
+	ret = ctrl_cmd_install(CTRL_NODE_ROOT, &cmd_stat_item);
 	if (ret)
 		goto err_vec;
 	ret = ctrl_cmd_install(CTRL_NODE_ROOT, &cmd_counter);
@@ -933,6 +1031,18 @@ struct ctrl_handle *ctrl_interface_setup_dynip(void *data,
 	return ctrl_interface_setup_dynip2(data, bind_addr, port, lookup, 0);
 }
 
+/*! Initializes CTRL interface using the configured bind addr/port.
+ *  \param[in] data Pointer which will be made available to each set_..() get_..() verify_..() control command function
+ *  \param[in] default_port TCP port number to bind to if not explicitly configured
+ *  \param[in] lookup Lookup function pointer, can be NULL
+ *  \param[in] node_count Number of CTRL nodes to allocate, 0 for default.
+ */
+struct ctrl_handle *ctrl_interface_setup2(void *data, uint16_t default_port, ctrl_cmd_lookup lookup,
+					  unsigned int node_count)
+{
+	return ctrl_interface_setup_dynip2(data, ctrl_vty_get_bind_addr(), ctrl_vty_get_bind_port(default_port), lookup,
+					   node_count);
+}
 
 /*! Install a lookup helper function for control nodes
  *  This function is used by e.g. library code to install lookup helpers
