@@ -163,9 +163,6 @@ struct gprs_ns2_inst {
 	/*! workaround for rate counter until rate counter accepts char str as index */
 	uint32_t nsvc_rate_ctr_idx;
 	uint32_t bind_rate_ctr_idx;
-
-	/*! libmnl netlink socket for link state monitoring */
-	struct osmo_mnl *linkmon_mnl;
 };
 
 
@@ -271,6 +268,9 @@ struct gprs_ns2_vc {
 	/*! recursive anchor */
 	bool freed;
 
+	/*! if blocked by O&M/vty */
+	bool om_blocked;
+
 	/*! when the NSVC became alive or dead */
 	struct timespec ts_alive_change;
 };
