@@ -3,6 +3,7 @@
 
 /* (C) 2020 by sysmocom - s.f.m.c. GmbH <info@sysmocom.de>
  * Author: Alexander Couzens <lynxis@fe80.eu>
+ * (C) 2021 by Harald Welte <laforge@osmocom.org>
  *
  * All Rights Reserved
  *
@@ -59,12 +60,14 @@ static struct gprs_ns2_inst *vty_nsi = NULL;
 static struct osmo_fr_network *vty_fr_network = NULL;
 static struct llist_head binds;
 static struct llist_head nses;
+static struct llist_head ip_sns_default_binds;
 
 struct vty_bind {
 	struct llist_head list;
 	const char *name;
 	enum gprs_ns2_ll ll;
 	int dscp;
+	uint8_t priority;
 	bool accept_ipaccess;
 	bool accept_sns;
 	uint8_t ip_sns_sig_weight;
@@ -98,6 +101,7 @@ static const struct value_string gprs_ns_timer_strs[] = {
 	{ 7, "tsns-prov" },
 	{ 8, "tsns-size-retries" },
 	{ 9, "tsns-config-retries" },
+	{10, "tsns-procedures-retries" },
 	{ 0, NULL }
 };
 
@@ -138,7 +142,7 @@ static struct vty_bind *vty_bind_alloc(const char *name)
 
 	vbind->ip_sns_sig_weight = 1;
 	vbind->ip_sns_data_weight = 1;
-	llist_add(&vbind->list, &binds);
+	llist_add_tail(&vbind->list, &binds);
 	return vbind;
 }
 
@@ -169,7 +173,7 @@ static struct vty_nse *vty_nse_alloc(uint16_t nsei)
 
 	vnse->nsei = nsei;
 	INIT_LLIST_HEAD(&vnse->binds);
-	llist_add(&vnse->list, &nses);
+	llist_add_tail(&vnse->list, &nses);
 	return vnse;
 }
 
@@ -214,6 +218,7 @@ static int vty_nse_remove_vbind(struct vty_nse *vnse, struct vty_bind *vbind)
 		if (vnse_bind->vbind == vbind) {
 			llist_del(&vnse_bind->list);
 			talloc_free(vnse_bind);
+			return 0;
 		}
 	}
 
@@ -270,15 +275,19 @@ DEFUN(cfg_ns_timer, cfg_ns_timer_cmd,
 }
 
 DEFUN(cfg_ns_nsei, cfg_ns_nsei_cmd,
-      "nse <0-65535>",
+      "nse <0-65535> [ip-sns-role-sgsn]",
       "Persistent NS Entity\n"
       "NS Entity ID (NSEI)\n"
+      "Create NSE in SGSN role (default: BSS)\n"
       )
 {
 	struct gprs_ns2_nse *nse;
 	struct vty_nse *vnse;
 	uint16_t nsei = atoi(argv[0]);
+	bool sgsn_role = false;
 	bool free_vnse = false;
+	if (argc > 1 && !strcmp(argv[1], "ip-sns-role-sgsn"))
+		sgsn_role = true;
 
 	vnse = vty_nse_by_nsei(nsei);
 	if (!vnse) {
@@ -292,7 +301,8 @@ DEFUN(cfg_ns_nsei, cfg_ns_nsei_cmd,
 
 	nse = gprs_ns2_nse_by_nsei(vty_nsi, nsei);
 	if (!nse) {
-		nse = gprs_ns2_create_nse(vty_nsi, nsei, GPRS_NS2_LL_UNDEF, GPRS_NS2_DIALECT_UNDEF);
+		nse = gprs_ns2_create_nse2(vty_nsi, nsei, GPRS_NS2_LL_UNDEF, GPRS_NS2_DIALECT_UNDEF,
+					   sgsn_role);
 		if (!nse) {
 			vty_out(vty, "Failed to create NSE!%s", VTY_NEWLINE);
 			goto err;
@@ -352,9 +362,9 @@ DEFUN(cfg_no_ns_nsei, cfg_no_ns_nsei_cmd,
 /* TODO: add fr/gre */
 DEFUN(cfg_ns_bind, cfg_ns_bind_cmd,
       "bind (fr|udp) ID",
-      "Binding\n"
+      "Configure local Bind\n"
       "Frame Relay\n" "UDP/IP\n"
-      "a unique identifier for this bind to reference NS-VCs\n"
+      "Unique identifier for this bind (to reference from NS-VCs, NSEs, ...)\n"
       )
 {
 	const char *nstype = argv[0];
@@ -398,8 +408,8 @@ DEFUN(cfg_ns_bind, cfg_ns_bind_cmd,
 DEFUN(cfg_no_ns_bind, cfg_no_ns_bind_cmd,
       "no bind ID",
       NO_STR
-      "Delete a binding\n"
-      "a unique identifier for this bind to reference NS-VCs\n"
+      "Delete a bind\n"
+      "Unique identifier for this bind\n"
       )
 {
 	struct vty_bind *vbind;
@@ -414,7 +424,7 @@ DEFUN(cfg_no_ns_bind, cfg_no_ns_bind_cmd,
 	vty_bind_free(vbind);
 	bind = gprs_ns2_bind_by_name(vty_nsi, name);
 	if (bind)
-		bind->driver->free_bind(bind);
+		gprs_ns2_free_bind(bind);
 	return CMD_SUCCESS;
 }
 
@@ -457,8 +467,12 @@ static void config_write_vbind(struct vty *vty, struct vty_bind *vbind)
 		}
 		if (vbind->accept_ipaccess)
 			vty_out(vty, "  accept-ipaccess%s", VTY_NEWLINE);
+		if (vbind->accept_sns)
+			vty_out(vty, "  accept-dynamic-ip-sns%s", VTY_NEWLINE);
 		if (vbind->dscp)
 			vty_out(vty, "  dscp %u%s", vbind->dscp, VTY_NEWLINE);
+		if (vbind->priority)
+			vty_out(vty, "  socket-priority %u%s", vbind->priority, VTY_NEWLINE);
 		vty_out(vty, "  ip-sns signalling-weight %u data-weight %u%s",
 			vbind->ip_sns_sig_weight, vbind->ip_sns_data_weight, VTY_NEWLINE);
 		break;
@@ -526,7 +540,8 @@ static void _config_write_ns_nse(struct vty *vty, struct gprs_ns2_nse *nse)
 
 	OSMO_ASSERT(vnse);
 
-	vty_out(vty, " nse %u%s", nse->nsei, VTY_NEWLINE);
+	vty_out(vty, " nse %u%s%s", nse->nsei,
+		nse->ip_sns_role_sgsn ? " ip-sns-role-sgsn" : "", VTY_NEWLINE);
 	switch (nse->dialect) {
 	case GPRS_NS2_DIALECT_SNS:
 		ns2_sns_write_vty(vty, nse);
@@ -569,6 +584,7 @@ static int config_write_ns_bind(struct vty *vty)
 
 static int config_write_ns(struct vty *vty)
 {
+	struct vty_nse_bind *vbind;
 	unsigned int i;
 	int ret;
 
@@ -583,6 +599,10 @@ static int config_write_ns(struct vty *vty)
 	if (ret)
 		return ret;
 
+	llist_for_each_entry(vbind, &ip_sns_default_binds, list) {
+		vty_out(vty, " ip-sns-default bind %s%s", vbind->vbind->name, VTY_NEWLINE);
+	}
+
 	ret = config_write_ns_nse(vty);
 	if (ret)
 		return ret;
@@ -599,9 +619,9 @@ static struct cmd_node ns_bind_node = {
 
 DEFUN(cfg_ns_bind_listen, cfg_ns_bind_listen_cmd,
       "listen " VTY_IPV46_CMD " <1-65535>",
-      "Binding\n"
-      "IPv4 Address\n" "IPv6 Address\n"
-      "Port\n"
+      "Configure local IP + Port of this bind\n"
+      "Local IPv4 Address\n" "Local IPv6 Address\n"
+      "Local UDP Port\n"
       )
 {
 	struct vty_bind *vbind = vty->index;
@@ -659,12 +679,12 @@ DEFUN(cfg_no_ns_bind_listen, cfg_no_ns_bind_listen_cmd,
 		return CMD_ERR_NOTHING_TODO;
 
 	OSMO_ASSERT(bind->ll == GPRS_NS2_LL_UDP);
-	bind->driver->free_bind(bind);
+	gprs_ns2_free_bind(bind);
 	return CMD_SUCCESS;
 }
 
 DEFUN(cfg_ns_bind_dscp, cfg_ns_bind_dscp_cmd,
-      "dscp <0-255>",
+      "dscp <0-63>",
       "Set DSCP/TOS on the UDP socket\n" "DSCP Value\n")
 {
 	struct vty_bind *vbind = vty->index;
@@ -707,6 +727,28 @@ DEFUN(cfg_no_ns_bind_dscp, cfg_no_ns_bind_dscp_cmd,
 	return CMD_SUCCESS;
 }
 
+DEFUN(cfg_ns_bind_priority, cfg_ns_bind_priority_cmd,
+      "socket-priority <0-255>",
+      "Set socket priority on the UDP socket\n" "Priority Value (>6 requires CAP_NET_ADMIN)\n")
+{
+	struct vty_bind *vbind = vty->index;
+	struct gprs_ns2_vc_bind *bind;
+	uint8_t prio = atoi(argv[0]);
+
+	if (vbind->ll != GPRS_NS2_LL_UDP) {
+		vty_out(vty, "dscp can be only used with UDP bind%s",
+			VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	vbind->priority = prio;
+	bind = gprs_ns2_bind_by_name(vty_nsi, vbind->name);
+	if (bind)
+		gprs_ns2_ip_bind_set_priority(bind, prio);
+
+	return CMD_SUCCESS;
+}
+
 DEFUN(cfg_ns_bind_ipaccess, cfg_ns_bind_ipaccess_cmd,
       "accept-ipaccess",
       "Allow to create dynamic NS Entity by NS Reset PDU on UDP (ip.access style)\n"
@@ -752,6 +794,51 @@ DEFUN(cfg_no_ns_bind_ipaccess, cfg_no_ns_bind_ipaccess_cmd,
 	return CMD_SUCCESS;
 }
 
+DEFUN(cfg_ns_bind_accept_sns, cfg_ns_bind_accept_sns_cmd,
+      "accept-dynamic-ip-sns",
+      "Allow to create dynamic NS Entities by IP-SNS PDUs\n"
+      )
+{
+	struct vty_bind *vbind = vty->index;
+	struct gprs_ns2_vc_bind *bind;
+
+	if (vbind->ll != GPRS_NS2_LL_UDP) {
+		vty_out(vty, "accept-dynamic-ip-sns can be only used with UDP bind%s",
+			VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	vbind->accept_sns = true;
+	bind = gprs_ns2_bind_by_name(vty_nsi, vbind->name);
+	if (bind)
+		bind->accept_sns = true;
+
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_no_ns_bind_accept_sns, cfg_no_ns_bind_accept_sns_cmd,
+      "no accept-dynamic-ip-sns",
+      NO_STR
+      "Disable dynamic creation of NS Entities by IP-SNS PDUs\n"
+      )
+{
+	struct vty_bind *vbind = vty->index;
+	struct gprs_ns2_vc_bind *bind;
+
+	if (vbind->ll != GPRS_NS2_LL_UDP) {
+		vty_out(vty, "no accept-dynamic-ip-sns can be only used with UDP bind%s",
+			VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	vbind->accept_sns = false;
+	bind = gprs_ns2_bind_by_name(vty_nsi, vbind->name);
+	if (bind)
+		bind->accept_sns = false;
+
+	return CMD_SUCCESS;
+}
+
 DEFUN(cfg_ns_bind_ip_sns_weight, cfg_ns_bind_ip_sns_weight_cmd,
       "ip-sns signalling-weight <0-254> data-weight <0-254>",
       "IP SNS\n"
@@ -853,7 +940,7 @@ DEFUN(cfg_no_ns_bind_fr, cfg_no_ns_bind_fr_cmd,
 		return CMD_WARNING;
 	}
 
-	bind->driver->free_bind(bind);
+	gprs_ns2_free_bind(bind);
 	return CMD_SUCCESS;
 }
 
@@ -900,7 +987,7 @@ DEFUN(cfg_ns_nse_nsvc_fr, cfg_ns_nse_nsvc_fr_cmd,
 	}
 
 	if (nse->dialect == GPRS_NS2_DIALECT_UNDEF) {
-		nse->dialect = GPRS_NS2_DIALECT_STATIC_RESETBLOCK;
+		ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_STATIC_RESETBLOCK);
 		dialect_modified = true;
 	}
 
@@ -935,7 +1022,7 @@ err:
 	if (ll_modified)
 		nse->ll = GPRS_NS2_LL_UNDEF;
 	if (dialect_modified)
-		nse->dialect = GPRS_NS2_DIALECT_UNDEF;
+		ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_UNDEF);
 
 	return CMD_WARNING;
 }
@@ -986,7 +1073,7 @@ DEFUN(cfg_no_ns_nse_nsvc_fr_dlci, cfg_no_ns_nse_nsvc_fr_dlci_cmd,
 	gprs_ns2_free_nsvc(nsvc);
 	if (llist_empty(&nse->nsvc)) {
 		nse->ll = GPRS_NS2_LL_UNDEF;
-		nse->dialect = GPRS_NS2_DIALECT_UNDEF;
+		ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_UNDEF);
 	}
 
 	return CMD_SUCCESS;
@@ -1032,7 +1119,7 @@ DEFUN(cfg_no_ns_nse_nsvci, cfg_no_ns_nse_nsvci_cmd,
 	gprs_ns2_free_nsvc(nsvc);
 	if (llist_empty(&nse->nsvc)) {
 		nse->ll = GPRS_NS2_LL_UNDEF;
-		nse->dialect = GPRS_NS2_DIALECT_UNDEF;
+		ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_UNDEF);
 	}
 
 	return CMD_SUCCESS;
@@ -1056,7 +1143,7 @@ static int ns_nse_nsvc_udp_cmds(struct vty *vty, const char *bind_name, const ch
 	}
 
 	if (nse->dialect == GPRS_NS2_DIALECT_UNDEF) {
-		nse->dialect = GPRS_NS2_DIALECT_STATIC_ALIVE;
+		ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_STATIC_ALIVE);
 		dialect_modified = true;
 	}
 
@@ -1117,7 +1204,7 @@ err:
 	if (ll_modified)
 		nse->ll = GPRS_NS2_LL_UNDEF;
 	if (dialect_modified)
-		nse->dialect = GPRS_NS2_DIALECT_UNDEF;
+		ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_UNDEF);
 	return CMD_WARNING;
 }
 
@@ -1232,7 +1319,7 @@ DEFUN(cfg_no_ns_nse_nsvc_udp, cfg_no_ns_nse_nsvc_udp_cmd,
 	gprs_ns2_free_nsvc(nsvc);
 	if (llist_empty(&nse->nsvc)) {
 		nse->ll = GPRS_NS2_LL_UNDEF;
-		nse->dialect = GPRS_NS2_DIALECT_UNDEF;
+		ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_UNDEF);
 	}
 
 	return CMD_SUCCESS;
@@ -1267,7 +1354,7 @@ DEFUN(cfg_ns_nse_nsvc_ipa, cfg_ns_nse_nsvc_ipa_cmd,
 	}
 
 	if (nse->dialect == GPRS_NS2_DIALECT_UNDEF) {
-		nse->dialect = GPRS_NS2_DIALECT_IPACCESS;
+		ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_IPACCESS);
 		dialect_modified = true;
 	}
 
@@ -1317,7 +1404,7 @@ err:
 	if (ll_modified)
 		nse->ll = GPRS_NS2_LL_UNDEF;
 	if (dialect_modified)
-		nse->dialect = GPRS_NS2_DIALECT_UNDEF;
+		ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_UNDEF);
 	return CMD_WARNING;
 }
 
@@ -1407,7 +1494,7 @@ DEFUN(cfg_no_ns_nse_nsvc_ipa, cfg_no_ns_nse_nsvc_ipa_cmd,
 	gprs_ns2_free_nsvc(nsvc);
 	if (llist_empty(&nse->nsvc)) {
 		nse->ll = GPRS_NS2_LL_UNDEF;
-		nse->dialect = GPRS_NS2_DIALECT_UNDEF;
+		ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_UNDEF);
 	}
 
 	return CMD_SUCCESS;
@@ -1436,12 +1523,8 @@ DEFUN(cfg_ns_nse_ip_sns_remote, cfg_ns_nse_ip_sns_remote_cmd,
 	}
 
 	if (nse->dialect == GPRS_NS2_DIALECT_UNDEF) {
-		char sns[16];
-		snprintf(sns, sizeof(sns), "NSE%05u-SNS", nse->nsei);
-		nse->bss_sns_fi = ns2_sns_bss_fsm_alloc(nse, sns);
-		if (!nse->bss_sns_fi)
+		if (ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_SNS) < 0)
 			goto err;
-		nse->dialect = GPRS_NS2_DIALECT_SNS;
 		dialect_modified = true;
 	}
 
@@ -1481,7 +1564,7 @@ err:
 	if (ll_modified)
 		nse->ll = GPRS_NS2_LL_UNDEF;
 	if (dialect_modified)
-		nse->dialect = GPRS_NS2_DIALECT_UNDEF;
+		ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_UNDEF);
 	return CMD_WARNING;
 }
 
@@ -1528,19 +1611,106 @@ DEFUN(cfg_no_ns_nse_ip_sns_remote, cfg_no_ns_nse_ip_sns_remote_cmd,
 		return CMD_SUCCESS;
 	} else {
 		/* clean up nse to allow other nsvc commands */
-		osmo_fsm_inst_term(nse->bss_sns_fi, OSMO_FSM_TERM_REQUEST, NULL);
-		nse->bss_sns_fi = NULL;
+		ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_UNDEF);
 		nse->ll = GPRS_NS2_LL_UNDEF;
-		nse->dialect = GPRS_NS2_DIALECT_UNDEF;
 	}
 
 	return CMD_SUCCESS;
 }
 
+/* add all IP-SNS default binds to the given NSE */
+int ns2_sns_add_sns_default_binds(struct gprs_ns2_nse *nse)
+{
+	struct vty_nse_bind *vnse_bind;
+	int count = 0;
+
+	OSMO_ASSERT(nse->ll == GPRS_NS2_LL_UDP);
+	OSMO_ASSERT(nse->dialect == GPRS_NS2_DIALECT_SNS);
+
+	llist_for_each_entry(vnse_bind, &ip_sns_default_binds, list) {
+		struct gprs_ns2_vc_bind *bind = gprs_ns2_bind_by_name(vty_nsi, vnse_bind->vbind->name);
+		/* the bind might not yet created because "listen" is missing. */
+		if (!bind)
+			continue;
+		gprs_ns2_sns_add_bind(nse, bind);
+		count++;
+	}
+	return count;
+}
+
+DEFUN(cfg_ns_ip_sns_default_bind, cfg_ns_ip_sns_default_bind_cmd,
+      "ip-sns-default bind ID",
+      "Defaults for dynamically created NSEs created by IP-SNS in SGSN role\n"
+      "IP SNS binds\n"
+      "Name of NS udp bind whose IP endpoint will be used as IP-SNS local endpoint. Can be given multiple times.\n")
+{
+	struct vty_bind *vbind;
+	struct vty_nse_bind *vnse_bind;
+	const char *name = argv[0];
+
+	vbind = vty_bind_by_name(name);
+	if (!vbind) {
+		vty_out(vty, "Can not find the given bind '%s'%s", name, VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	if (vbind->ll != GPRS_NS2_LL_UDP) {
+		vty_out(vty, "ip-sns-default bind can only be used with UDP bind%s", VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	llist_for_each_entry(vnse_bind, &ip_sns_default_binds, list) {
+		if (vnse_bind->vbind == vbind)
+			return CMD_SUCCESS;
+	}
+
+	vnse_bind = talloc(vty_nsi, struct vty_nse_bind);
+	if (!vnse_bind)
+		return CMD_WARNING;
+	vnse_bind->vbind = vbind;
+
+	llist_add_tail(&vnse_bind->list, &ip_sns_default_binds);
+
+	return CMD_SUCCESS;
+}
+
+DEFUN(cfg_no_ns_ip_sns_default_bind, cfg_no_ns_ip_sns_default_bind_cmd,
+      "no ip-sns-default bind ID",
+      NO_STR "Defaults for dynamically created NSEs created by IP-SNS in SGSN role\n"
+      "IP SNS binds\n"
+      "Name of NS udp bind whose IP endpoint will be removed as IP-SNS local endpoint.\n")
+{
+	struct vty_bind *vbind;
+	struct vty_nse_bind *vnse_bind;
+	const char *name = argv[0];
+
+	vbind = vty_bind_by_name(name);
+	if (!vbind) {
+		vty_out(vty, "Can not find the given bind '%s'%s", name, VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	if (vbind->ll != GPRS_NS2_LL_UDP) {
+		vty_out(vty, "ip-sns-default bind can only be used with UDP bind%s", VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	llist_for_each_entry(vnse_bind, &ip_sns_default_binds, list) {
+		if (vnse_bind->vbind == vbind) {
+			llist_del(&vnse_bind->list);
+			talloc_free(vnse_bind);
+			return CMD_SUCCESS;
+		}
+	}
+
+	vty_out(vty, "Bind '%s' was not an ip-sns-default bind%s", name, VTY_NEWLINE);
+	return CMD_WARNING;
+}
+
 DEFUN(cfg_ns_nse_ip_sns_bind, cfg_ns_nse_ip_sns_bind_cmd,
       "ip-sns-bind BINDID",
       "IP SNS binds\n"
-      "A udp bind which this SNS will be used. The bind must be already exists. Can be given multiple times.\n")
+      "Name of NS udp bind whose IP endpoint will be used as IP-SNS local endpoint. Can be given multiple times.\n")
 {
 	struct gprs_ns2_nse *nse = vty->index;
 	struct gprs_ns2_vc_bind *bind;
@@ -1557,12 +1727,8 @@ DEFUN(cfg_ns_nse_ip_sns_bind, cfg_ns_nse_ip_sns_bind_cmd,
 	}
 
 	if (nse->dialect == GPRS_NS2_DIALECT_UNDEF) {
-		char sns[16];
-		snprintf(sns, sizeof(sns), "NSE%05u-SNS", nse->nsei);
-		nse->bss_sns_fi = ns2_sns_bss_fsm_alloc(nse, sns);
-		if (!nse->bss_sns_fi)
+		if (ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_SNS) < 0)
 			goto err;
-		nse->dialect = GPRS_NS2_DIALECT_SNS;
 		dialect_modified = true;
 	}
 
@@ -1632,7 +1798,7 @@ err:
 	if (ll_modified)
 		nse->ll = GPRS_NS2_LL_UNDEF;
 	if (dialect_modified)
-		nse->dialect = GPRS_NS2_DIALECT_UNDEF;
+		ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_UNDEF);
 
 	return CMD_WARNING;
 }
@@ -1641,7 +1807,7 @@ DEFUN(cfg_no_ns_nse_ip_sns_bind, cfg_no_ns_nse_ip_sns_bind_cmd,
       "no ip-sns-bind BINDID",
       NO_STR
       "IP SNS binds\n"
-      "A udp bind which this SNS will be used.\n")
+      "Name of NS udp bind whose IP endpoint will not be used as IP-SNS local endpoint\n")
 {
 	struct gprs_ns2_nse *nse = vty->index;
 	struct gprs_ns2_vc_bind *bind;
@@ -1698,53 +1864,62 @@ DEFUN(cfg_no_ns_nse_ip_sns_bind, cfg_no_ns_nse_ip_sns_bind_cmd,
 
 	if (!vty_nse_check_sns(nse)) {
 		/* clean up nse to allow other nsvc commands */
-		osmo_fsm_inst_term(nse->bss_sns_fi, OSMO_FSM_TERM_REQUEST, NULL);
-		nse->bss_sns_fi = NULL;
+		ns2_nse_set_dialect(nse, GPRS_NS2_DIALECT_UNDEF);
 		nse->ll = GPRS_NS2_LL_UNDEF;
-		nse->dialect = GPRS_NS2_DIALECT_UNDEF;
 	}
 
 	return CMD_SUCCESS;
 }
 
 /* non-config commands */
-static void dump_nsvc(struct vty *vty, struct gprs_ns2_vc *nsvc, bool stats)
+void ns2_vty_dump_nsvc(struct vty *vty, struct gprs_ns2_vc *nsvc, bool stats)
 {
-	char nsvci_str[32];
-
 	if (nsvc->nsvci_is_valid)
-		snprintf(nsvci_str, sizeof(nsvci_str), "%05u", nsvc->nsvci);
+		vty_out(vty, "   NSVCI %05u: %s %s %s %s %ssince ", nsvc->nsvci,
+			osmo_fsm_inst_state_name(nsvc->fi),
+			nsvc->persistent ? "PERSIST" : "DYNAMIC",
+			gprs_ns2_ll_str(nsvc),
+			ns2_vc_is_unblocked(nsvc) ? "ALIVE" : "DEAD",
+			nsvc->om_blocked ? "(blocked by O&M/vty) " :
+				!ns2_vc_is_unblocked(nsvc) ? "(cause: remote) " : "");
 	else
-		snprintf(nsvci_str, sizeof(nsvci_str), "none");
+		vty_out(vty, "   %s %s sig_weight=%u data_weight=%u %s %s %ssince ",
+			osmo_fsm_inst_state_name(nsvc->fi),
+			nsvc->persistent ? "PERSIST" : "DYNAMIC",
+			nsvc->sig_weight, nsvc->data_weight,
+			gprs_ns2_ll_str(nsvc),
+			ns2_vc_is_unblocked(nsvc) ? "ALIVE" : "DEAD",
+				!ns2_vc_is_unblocked(nsvc) ? "(cause: remote) " : "");
 
-	vty_out(vty, " NSVCI %s: %s %s data_weight=%u sig_weight=%u %s%s", nsvci_str,
-		osmo_fsm_inst_state_name(nsvc->fi),
-		nsvc->persistent ? "PERSIST" : "DYNAMIC",
-		nsvc->data_weight, nsvc->sig_weight,
-		gprs_ns2_ll_str(nsvc), VTY_NEWLINE);
+	vty_out_uptime(vty, &nsvc->ts_alive_change);
+	vty_out_newline(vty);
 
 	if (stats) {
-		vty_out_rate_ctr_group(vty, "  ", nsvc->ctrg);
-		vty_out_stat_item_group(vty, "  ", nsvc->statg);
+		vty_out_rate_ctr_group(vty, "    ", nsvc->ctrg);
+		vty_out_stat_item_group(vty, "    ", nsvc->statg);
 	}
 }
 
 static void dump_nse(struct vty *vty, const struct gprs_ns2_nse *nse, bool stats, bool persistent_only)
 {
 	struct gprs_ns2_vc *nsvc;
+	unsigned int nsvcs = 0;
 
-	vty_out(vty, "NSEI %05u: %s, %s%s", nse->nsei, gprs_ns2_lltype_str(nse->ll),
-		nse->alive ? "ALIVE" : "DEAD", VTY_NEWLINE);
+	if (persistent_only && !nse->persistent)
+		return;
+
+	vty_out(vty, "NSEI %05u: %s, %s since ", nse->nsei, gprs_ns2_lltype_str(nse->ll),
+		nse->alive ? "ALIVE" : "DEAD");
+	vty_out_uptime(vty, &nse->ts_alive_change);
+	vty_out_newline(vty);
 
 	ns2_sns_dump_vty(vty, " ", nse, stats);
 	llist_for_each_entry(nsvc, &nse->nsvc, list) {
-		if (persistent_only) {
-			if (nsvc->persistent)
-				dump_nsvc(vty, nsvc, stats);
-		} else {
-			dump_nsvc(vty, nsvc, stats);
-		}
+		nsvcs++;
 	}
+	vty_out(vty, "  %u NS-VC:%s", nsvcs, VTY_NEWLINE);
+	llist_for_each_entry(nsvc, &nse->nsvc, list)
+		ns2_vty_dump_nsvc(vty, nsvc, stats);
 }
 
 static void dump_bind(struct vty *vty, const struct gprs_ns2_vc_bind *bind, bool stats)
@@ -1783,6 +1958,8 @@ DEFUN_HIDDEN(show_ns, show_ns_cmd, "show ns",
 {
 	dump_ns_entities(vty, vty_nsi, false, false);
 	dump_ns_bind(vty, vty_nsi, false);
+	if (vty_fr_network && llist_count(&vty_fr_network->links))
+		osmo_fr_network_dump_vty(vty, vty_fr_network);
 	return CMD_SUCCESS;
 }
 
@@ -1852,7 +2029,7 @@ DEFUN(show_nse, show_nse_cmd, "show ns (nsei|nsvc) <0-65535> [stats]",
 			return CMD_WARNING;
 		}
 
-		dump_nsvc(vty, nsvc, show_stats);
+		ns2_vty_dump_nsvc(vty, nsvc, show_stats);
 	}
 
 	return CMD_SUCCESS;
@@ -1895,15 +2072,42 @@ DEFUN_HIDDEN(nsvc_force_unconf, nsvc_force_unconf_cmd,
 	return CMD_SUCCESS;
 }
 
+DEFUN(nse_restart_sns, nse_restart_sns_cmd,
+      "nse <0-65535> restart-sns",
+      "NSE specific commands\n"
+      "NS Entity ID (NSEI)\n"
+      "Restart SNS procedure\n")
+{
+	struct gprs_ns2_inst *nsi = vty_nsi;
+	struct gprs_ns2_nse *nse;
+
+	uint16_t id = atoi(argv[0]);
+	nse = gprs_ns2_nse_by_nsei(nsi, id);
+	if (!nse) {
+		vty_out(vty, "Could not find NSE for NSEI %u%s", id, VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	if (nse->dialect != GPRS_NS2_DIALECT_SNS) {
+		vty_out(vty, "Given NSEI %u doesn't use IP-SNS%s", id, VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
+	gprs_ns2_free_nsvcs(nse);
+	return CMD_SUCCESS;
+}
+
 DEFUN(nsvc_block, nsvc_block_cmd,
-      "nsvc <0-65535> (block|unblock)",
+      "nsvc <0-65535> (block|unblock|reset)",
       "NS Virtual Connection\n"
       NSVCI_STR
       "Block a NSVC. As cause code O&M intervention will be used.\n"
-      "Unblock a NSVC. As cause code O&M intervention will be used.\n")
+      "Unblock a NSVC. As cause code O&M intervention will be used.\n"
+      "Reset a NSVC. As cause code O&M intervention will be used.\n")
 {
 	struct gprs_ns2_inst *nsi = vty_nsi;
 	struct gprs_ns2_vc *nsvc;
+	int rc;
 
 	uint16_t id = atoi(argv[0]);
 
@@ -1914,9 +2118,34 @@ DEFUN(nsvc_block, nsvc_block_cmd,
 	}
 
 	if (!strcmp(argv[1], "block")) {
-		ns2_vc_block(nsvc);
+		rc = ns2_vc_block(nsvc);
+		switch (rc) {
+		case 0:
+			vty_out(vty, "The NS-VC %05u will be blocked.%s", id, VTY_NEWLINE);
+			return CMD_SUCCESS;
+		case -EALREADY:
+			vty_out(vty, "The NS-VC %05u is already blocked.%s", id, VTY_NEWLINE);
+			return CMD_ERR_NOTHING_TODO;
+		default:
+			vty_out(vty, "An unknown error %d happend on NS-VC %05u.%s", rc, id, VTY_NEWLINE);
+			return CMD_WARNING;
+		}
+	} else if (!strcmp(argv[1], "unblock")) {
+		rc = ns2_vc_unblock(nsvc);
+		switch (rc) {
+		case 0:
+			vty_out(vty, "The NS-VC %05u will be unblocked.%s", id, VTY_NEWLINE);
+			return CMD_SUCCESS;
+		case -EALREADY:
+			vty_out(vty, "The NS-VC %05u is already unblocked.%s", id, VTY_NEWLINE);
+			return CMD_ERR_NOTHING_TODO;
+		default:
+			vty_out(vty, "An unknown error %d happend on NS-VC %05u.%s", rc, id, VTY_NEWLINE);
+			return CMD_WARNING;
+		}
 	} else {
-		ns2_vc_unblock(nsvc);
+		ns2_vc_reset(nsvc);
+		vty_out(vty, "The NS-VC %05u has been resetted.%s", id, VTY_NEWLINE);
 	}
 
 	return CMD_SUCCESS;
@@ -2019,6 +2248,7 @@ int gprs_ns2_vty_init_reduced(struct gprs_ns2_inst *nsi)
 	vty_nsi = nsi;
 	INIT_LLIST_HEAD(&binds);
 	INIT_LLIST_HEAD(&nses);
+	INIT_LLIST_HEAD(&ip_sns_default_binds);
 
 	vty_fr_network = osmo_fr_network_alloc(nsi);
 	if (!vty_fr_network)
@@ -2034,6 +2264,7 @@ int gprs_ns2_vty_init_reduced(struct gprs_ns2_inst *nsi)
 
 	install_lib_element(ENABLE_NODE, &nsvc_force_unconf_cmd);
 	install_lib_element(ENABLE_NODE, &nsvc_block_cmd);
+	install_lib_element(ENABLE_NODE, &nse_restart_sns_cmd);
 
 	install_lib_element(CFG_LOG_NODE, &logging_fltr_nse_cmd);
 	install_lib_element(CFG_LOG_NODE, &logging_fltr_nsvc_cmd);
@@ -2058,17 +2289,22 @@ int gprs_ns2_vty_init(struct gprs_ns2_inst *nsi)
 	install_lib_element(L_NS_NODE, &cfg_ns_bind_cmd);
 	install_lib_element(L_NS_NODE, &cfg_no_ns_bind_cmd);
 
+	install_lib_element(L_NS_NODE, &cfg_ns_ip_sns_default_bind_cmd);
+	install_lib_element(L_NS_NODE, &cfg_no_ns_ip_sns_default_bind_cmd);
+
 	install_node(&ns_bind_node, NULL);
 	install_lib_element(L_NS_BIND_NODE, &cfg_ns_bind_listen_cmd);
 	install_lib_element(L_NS_BIND_NODE, &cfg_no_ns_bind_listen_cmd);
 	install_lib_element(L_NS_BIND_NODE, &cfg_ns_bind_dscp_cmd);
 	install_lib_element(L_NS_BIND_NODE, &cfg_no_ns_bind_dscp_cmd);
+	install_lib_element(L_NS_BIND_NODE, &cfg_ns_bind_priority_cmd);
 	install_lib_element(L_NS_BIND_NODE, &cfg_ns_bind_ip_sns_weight_cmd);
 	install_lib_element(L_NS_BIND_NODE, &cfg_ns_bind_ipaccess_cmd);
 	install_lib_element(L_NS_BIND_NODE, &cfg_no_ns_bind_ipaccess_cmd);
 	install_lib_element(L_NS_BIND_NODE, &cfg_ns_bind_fr_cmd);
 	install_lib_element(L_NS_BIND_NODE, &cfg_no_ns_bind_fr_cmd);
-	/* TODO: accept-ip-sns when SGSN SNS has been implemented */
+	install_lib_element(L_NS_BIND_NODE, &cfg_ns_bind_accept_sns_cmd);
+	install_lib_element(L_NS_BIND_NODE, &cfg_no_ns_bind_accept_sns_cmd);
 
 	install_node(&ns_nse_node, NULL);
 	install_lib_element(L_NS_NSE_NODE, &cfg_ns_nse_nsvc_fr_cmd);
