@@ -43,6 +43,9 @@ const struct value_string osmo_bts_features_descs[] = {
 	{ BTS_FEAT_BCCH_POWER_RED,	"BCCH carrier power reduction mode" },
 	{ BTS_FEAT_DYN_TS_SDCCH8,	"Dynamic Timeslot configuration as SDCCH8" },
 	{ BTS_FEAT_ACCH_TEMP_OVP,	"FACCH/SACCH Temporary overpower" },
+	{ BTS_FEAT_OSMUX,		"Osmux (Osmocom RTP multiplexing)" },
+	{ BTS_FEAT_VBS,			"Voice Broadcast Service" },
+	{ BTS_FEAT_VGCS,		"Voice Group Call Service" },
 	{ 0, NULL }
 };
 
@@ -77,5 +80,7 @@ const struct value_string osmo_bts_features_names[] = {
 	{ BTS_FEAT_ABIS_OSMO_PCU, "ABIS_OSMO_PCU" },
 	{ BTS_FEAT_BCCH_POWER_RED, "BCCH_PWR_RED" },
 	{ BTS_FEAT_DYN_TS_SDCCH8, "DYN_TS_SDCCH8" },
+	{ BTS_FEAT_ACCH_TEMP_OVP, "ACCH_TEMP_OVP" },
+	{ BTS_FEAT_OSMUX, "OSMUX" },
 	{}
 };
