@@ -100,13 +100,19 @@ struct msgb *gsm0808_create_layer3_2(const struct msgb *msg_l3, const struct osm
 		     msgb_l3len(msg_l3), msg_l3->l3h);
 
 	/* AoIP: add Codec List (BSS Supported) 3.2.2.103 */
-	if (scl)
-		gsm0808_enc_speech_codec_list(msg, scl);
+	if (scl) {
+		if (gsm0808_enc_speech_codec_list2(msg, scl) < 0)
+			goto exit_free;
+	}
 
 	/* push the bssmap header */
 	msg->l3h = msgb_tv_push(msg, BSSAP_MSG_BSS_MANAGEMENT, msgb_length(msg));
 
 	return msg;
+
+exit_free:
+	msgb_free(msg);
+	return NULL;
 }
 
 /*! Create "Complete L3 Info" for A, legacy implementation.
@@ -373,7 +379,7 @@ struct msgb *gsm0808_create_lcls_conn_ctrl(enum gsm0808_lcls_config config,
 	if (config != GSM0808_LCLS_CFG_NA)
 		msgb_tv_put(msg, GSM0808_IE_LCLS_CONFIG, config);
 	if (control != GSM0808_LCLS_CSC_NA)
-		msgb_tv_put(msg, GSM0808_IE_LCLS_CONFIG, control);
+		msgb_tv_put(msg, GSM0808_IE_LCLS_CONN_STATUS_CTRL, control);
 	msg->l3h = msgb_tv_push(msg, BSSAP_MSG_BSS_MANAGEMENT, msgb_length(msg));
 
 	return msg;
@@ -418,7 +424,7 @@ struct msgb *gsm0808_create_lcls_notification(enum gsm0808_lcls_status status, b
 
 /*! Create BSSMAP Classmark Request message
  *  \returns callee-allocated msgb with BSSMAP Classmark Request message */
-struct msgb *gsm0808_create_classmark_request()
+struct msgb *gsm0808_create_classmark_request(void)
 {
 	struct msgb *msg = msgb_alloc_headroom(BSSMAP_MSG_SIZE, BSSMAP_MSG_HEADROOM,
 					       "classmark-request");
@@ -530,8 +536,10 @@ struct msgb *gsm0808_create_ass2(const struct gsm0808_channel_type *ct,
 	}
 
 	/* AoIP: Codec List (MSC Preferred) 3.2.2.103 */
-	if (scl)
-		gsm0808_enc_speech_codec_list(msg, scl);
+	if (scl) {
+		if (gsm0808_enc_speech_codec_list2(msg, scl) < 0)
+			goto exit_free;
+	}
 
 	/* AoIP: Call Identifier 3.2.2.105 */
 	if (ci) {
@@ -552,6 +560,10 @@ struct msgb *gsm0808_create_ass2(const struct gsm0808_channel_type *ct,
 	    msgb_tv_push(msg, BSSAP_MSG_BSS_MANAGEMENT, msgb_length(msg));
 
 	return msg;
+
+exit_free:
+	msgb_free(msg);
+	return NULL;
 }
 
 /*! Create BSSMAP Assignment Request message, 3GPP TS 48.008 §3.2.1.1.
@@ -616,12 +628,16 @@ struct msgb *gsm0808_create_ass_compl2(uint8_t rr_cause, uint8_t chosen_channel,
 		gsm0808_enc_aoip_trasp_addr(msg, ss);
 
 	/* AoIP: Speech Codec (Chosen) 3.2.2.104 */
-	if (sc)
-		gsm0808_enc_speech_codec(msg, sc);
+	if (sc) {
+		if (gsm0808_enc_speech_codec2(msg, sc) < 0)
+			goto exit_free;
+	}
 
 	/* AoIP: add Codec List (BSS Supported) 3.2.2.103 */
-	if (scl)
-		gsm0808_enc_speech_codec_list(msg, scl);
+	if (scl) {
+		if (gsm0808_enc_speech_codec_list2(msg, scl) < 0)
+			goto exit_free;
+	}
 
 	/* FIXME: write LSA identifier 3.2.2.15 - see 3GPP TS 43.073 */
 
@@ -632,6 +648,10 @@ struct msgb *gsm0808_create_ass_compl2(uint8_t rr_cause, uint8_t chosen_channel,
 	msg->l3h = msgb_tv_push(msg, BSSAP_MSG_BSS_MANAGEMENT, msgb_length(msg));
 
 	return msg;
+
+exit_free:
+	msgb_free(msg);
+	return NULL;
 }
 
 /*! Create BSSMAP Assignment Completed message
@@ -693,13 +713,19 @@ struct msgb *gsm0808_create_ass_fail(uint8_t cause, const uint8_t *rr_cause,
 	/* Circuit pool list 3.2.2.46 */
 
 	/* AoIP: add Codec List (BSS Supported) 3.2.2.103 */
-	if (scl)
-		gsm0808_enc_speech_codec_list(msg, scl);
+	if (scl) {
+		if (gsm0808_enc_speech_codec_list2(msg, scl) < 0)
+			goto exit_free;
+	}
 
 	/* update the size */
 	msg->l3h = msgb_tv_push(msg, BSSAP_MSG_BSS_MANAGEMENT, msgb_length(msg));
 
 	return msg;
+
+exit_free:
+	msgb_free(msg);
+	return NULL;
 }
 
 /*! Create BSSMAP Assignment Failure message
@@ -992,8 +1018,10 @@ struct msgb *gsm0808_create_handover_request(const struct gsm0808_handover_reque
 	if (params->aoip_transport_layer)
 		gsm0808_enc_aoip_trasp_addr(msg, params->aoip_transport_layer);
 
-	if (params->codec_list_msc_preferred)
-		gsm0808_enc_speech_codec_list(msg, params->codec_list_msc_preferred);
+	if (params->codec_list_msc_preferred) {
+		if (gsm0808_enc_speech_codec_list2(msg, params->codec_list_msc_preferred) < 0)
+			goto exit_free;
+	}
 
 	if (params->call_id_present) {
 		uint8_t val[4];
@@ -1013,6 +1041,10 @@ struct msgb *gsm0808_create_handover_request(const struct gsm0808_handover_reque
 	msg->l3h = msgb_tv_push(msg, BSSAP_MSG_BSS_MANAGEMENT, msgb_length(msg));
 
 	return msg;
+
+exit_free:
+	msgb_free(msg);
+	return NULL;
 }
 
 /*! Create BSSMAP HANDOVER REQUEST ACKNOWLEDGE message, 3GPP TS 48.008 3.2.1.10.
@@ -1048,17 +1080,25 @@ struct msgb *gsm0808_create_handover_request_ack2(const struct gsm0808_handover_
 	/* AoIP: add Codec List (BSS Supported) 3.2.2.103.
 	 * (codec_list_bss_supported was added to struct gsm0808_handover_request_ack later than speech_codec_chosen
 	 * below, but it needs to come before it in the message coding). */
-	if (params->more_items && params->codec_list_bss_supported.len)
-		gsm0808_enc_speech_codec_list(msg, &params->codec_list_bss_supported);
+	if (params->more_items && params->codec_list_bss_supported.len) {
+		if (gsm0808_enc_speech_codec_list2(msg, &params->codec_list_bss_supported) < 0)
+			goto exit_free;
+	}
 
 	/* AoIP: Speech Codec (Chosen) 3.2.2.104 */
-	if (params->speech_codec_chosen_present)
-		gsm0808_enc_speech_codec(msg, &params->speech_codec_chosen);
+	if (params->speech_codec_chosen_present) {
+		if (gsm0808_enc_speech_codec2(msg, &params->speech_codec_chosen) < 0)
+			goto exit_free;
+	}
 
 	/* prepend header with final length */
 	msg->l3h = msgb_tv_push(msg, BSSAP_MSG_BSS_MANAGEMENT, msgb_length(msg));
 
 	return msg;
+
+exit_free:
+	msgb_free(msg);
+	return NULL;
 }
 
 /*! Same as gsm0808_create_handover_request_ack2() but with less parameters.
@@ -1110,7 +1150,7 @@ struct msgb *gsm0808_create_handover_command(const struct gsm0808_handover_comma
 /*! Create BSSMAP HANDOVER DETECT message, 3GPP TS 48.008 3.2.1.40.
  * Sent from the MT BSC back to the MSC when the MS has sent a handover RACH request and the MT BSC has
  * received the Handover Detect message. */
-struct msgb *gsm0808_create_handover_detect()
+struct msgb *gsm0808_create_handover_detect(void)
 {
 	struct msgb *msg;
 
@@ -1129,7 +1169,7 @@ struct msgb *gsm0808_create_handover_detect()
 
 /*! Create BSSMAP HANDOVER SUCCEEDED message, 3GPP TS 48.008 3.2.1.13.
  * Sent from the MSC back to the old BSS to notify that the MS has successfully accessed the new BSS. */
-struct msgb *gsm0808_create_handover_succeeded()
+struct msgb *gsm0808_create_handover_succeeded(void)
 {
 	struct msgb *msg;
 
@@ -1164,12 +1204,16 @@ struct msgb *gsm0808_create_handover_complete(const struct gsm0808_handover_comp
 		msgb_tlv_put(msg, GSM0808_IE_RR_CAUSE, 1, &params->rr_cause);
 
 	/* AoIP: Speech Codec (Chosen) 3.2.2.104 */
-	if (params->speech_codec_chosen_present)
-		gsm0808_enc_speech_codec(msg, &params->speech_codec_chosen);
+	if (params->speech_codec_chosen_present) {
+		if (gsm0808_enc_speech_codec2(msg, &params->speech_codec_chosen) < 0)
+			goto exit_free;
+	}
 
 	/* AoIP: add Codec List (BSS Supported) 3.2.2.103 */
-	if (params->codec_list_bss_supported.len)
-		gsm0808_enc_speech_codec_list(msg, &params->codec_list_bss_supported);
+	if (params->codec_list_bss_supported.len) {
+		if (gsm0808_enc_speech_codec_list2(msg, &params->codec_list_bss_supported) < 0)
+			goto exit_free;
+	}
 
 	/* Chosen Encryption Algorithm 3.2.2.44 */
 	if (params->chosen_encr_alg_present && params->chosen_encr_alg > 0)
@@ -1183,6 +1227,10 @@ struct msgb *gsm0808_create_handover_complete(const struct gsm0808_handover_comp
 	msg->l3h = msgb_tv_push(msg, BSSAP_MSG_BSS_MANAGEMENT, msgb_length(msg));
 
 	return msg;
+
+exit_free:
+	msgb_free(msg);
+	return NULL;
 }
 
 /*! Create BSSMAP HANDOVER FAILURE message, 3GPP TS 48.008 3.2.1.16.
@@ -1206,13 +1254,19 @@ struct msgb *gsm0808_create_handover_failure(const struct gsm0808_handover_failu
 		msgb_tlv_put(msg, GSM0808_IE_RR_CAUSE, 1, &params->rr_cause);
 
 	/* AoIP: add Codec List (BSS Supported) 3.2.2.103 */
-	if (params->codec_list_bss_supported.len)
-		gsm0808_enc_speech_codec_list(msg, &params->codec_list_bss_supported);
+	if (params->codec_list_bss_supported.len) {
+		if (gsm0808_enc_speech_codec_list2(msg, &params->codec_list_bss_supported) < 0)
+			goto exit_free;
+	}
 
 	/* prepend header with final length */
 	msg->l3h = msgb_tv_push(msg, BSSAP_MSG_BSS_MANAGEMENT, msgb_length(msg));
 
 	return msg;
+
+exit_free:
+	msgb_free(msg);
+	return NULL;
 }
 
 /*! Create BSSMAP HANDOVER PERFORMED message, 3GPP TS 48.008 3.2.1.25.
@@ -1248,8 +1302,10 @@ struct msgb *gsm0808_create_handover_performed(const struct gsm0808_handover_per
 		msgb_tv_put(msg, GSM0808_IE_SPEECH_VERSION, params->speech_version_chosen);
 
 	/* AoIP: Speech Codec (chosen) 3.2.2.104 */
-	if (params->speech_codec_chosen_present)
-		gsm0808_enc_speech_codec(msg, &params->speech_codec_chosen);
+	if (params->speech_codec_chosen_present) {
+		if (gsm0808_enc_speech_codec2(msg, &params->speech_codec_chosen) < 0)
+			goto exit_free;
+	}
 
 	/* LCLS-BSS-Status 3.2.2.119 */
 	if (params->lcls_bss_status_present)
@@ -1259,6 +1315,10 @@ struct msgb *gsm0808_create_handover_performed(const struct gsm0808_handover_per
 	msg->l3h = msgb_tv_push(msg, BSSAP_MSG_BSS_MANAGEMENT, msgb_length(msg));
 
 	return msg;
+
+exit_free:
+	msgb_free(msg);
+	return NULL;
 }
 
 /*! Create BSSMAP COMMON ID message, 3GPP TS 48.008 3.2.1.68.
@@ -1728,7 +1788,7 @@ static const struct value_string gsm0808_msgt_names[] = {
 	{ BSS_MAP_MSG_UPLINK_APP_DATA,		"UPLINK APP DATA" },
 
 	{ BSS_MAP_MSG_LCLS_CONNECT_CTRL,	"LCLS-CONNECT-CONTROL" },
-	{ BSS_MAP_MSG_LCLS_CONNECT_CTRL_ACK,	"CLS-CONNECT-CONTROL-ACK" },
+	{ BSS_MAP_MSG_LCLS_CONNECT_CTRL_ACK,	"LCLS-CONNECT-CONTROL-ACK" },
 	{ BSS_MAP_MSG_LCLS_NOTIFICATION,	"LCLS-NOTIFICATION" },
 
 	{ 0, NULL }
