@@ -78,7 +78,7 @@ uint8_t gsm0808_enc_cause(struct msgb *msg, uint16_t cause)
 /*! Encode TS 08.08 AoIP transport address IE
  *  \param[out] msg Message Buffer to which to append IE
  *  \param[in] ss Socket Address to be used in IE
- *  \returns number of bytes added to \a msg */
+ *  \returns number of bytes added to \a msg; 0 if msg is too small */
 uint8_t gsm0808_enc_aoip_trasp_addr(struct msgb *msg,
 				    const struct sockaddr_storage *ss)
 {
@@ -87,16 +87,25 @@ uint8_t gsm0808_enc_aoip_trasp_addr(struct msgb *msg,
 	struct sockaddr_in6 *sin6;
 	uint16_t port = 0;
 	uint8_t *ptr;
-	uint8_t *old_tail;
-	uint8_t *tlv_len;
+	const uint8_t len_tl = 2;
+	uint8_t len_v = sizeof(port);
 
-	OSMO_ASSERT(msg);
-	OSMO_ASSERT(ss);
 	OSMO_ASSERT(ss->ss_family == AF_INET || ss->ss_family == AF_INET6);
 
+	switch (ss->ss_family) {
+	case AF_INET:
+		len_v += IP_V4_ADDR_LEN;
+		break;
+	case AF_INET6:
+		len_v += IP_V6_ADDR_LEN;
+		break;
+	}
+
+	if (msgb_tailroom(msg) < len_tl + len_v)
+		return 0;
+
 	msgb_put_u8(msg, GSM0808_IE_AOIP_TRASP_ADDR);
-	tlv_len = msgb_put(msg,1);
-	old_tail = msg->tail;
+	msgb_put_u8(msg, len_v);
 
 	switch (ss->ss_family) {
 	case AF_INET:
@@ -114,9 +123,7 @@ uint8_t gsm0808_enc_aoip_trasp_addr(struct msgb *msg,
 	}
 
 	msgb_put_u16(msg, port);
-
-	*tlv_len = (uint8_t) (msg->tail - old_tail);
-	return *tlv_len + 2;
+	return len_tl + len_v;
 }
 
 /*! Decode TS 08.08 AoIP transport address IE
@@ -132,7 +139,6 @@ int gsm0808_dec_aoip_trasp_addr(struct sockaddr_storage *ss,
 	struct sockaddr_in6 sin6;
 	const uint8_t *old_elem = elem;
 
-	OSMO_ASSERT(ss);
 	if (!elem)
 		return -EINVAL;
 	if (len == 0)
@@ -180,7 +186,6 @@ int gsm0808_dec_aoip_trasp_addr(struct sockaddr_storage *ss,
  *  \returns number of bytes parsed */
 int gsm0808_dec_osmux_cid(uint8_t *cid, const uint8_t *elem, uint8_t len)
 {
-	OSMO_ASSERT(cid);
 	if (!elem)
 		return -EINVAL;
 	if (len != 1)
@@ -204,21 +209,21 @@ static void decode_lai(const uint8_t *data, struct osmo_location_area_id *decode
 	gsm48_decode_lai2(&lai, decoded);
 }
 
-/* Helper function for gsm0808_enc_speech_codec()
- * and gsm0808_enc_speech_codec_list() */
-static uint8_t enc_speech_codec(struct msgb *msg,
-				const struct gsm0808_speech_codec *sc)
+/* Helper function for gsm0808_enc_speech_codec[_list]().
+ * Returns number of bytes appended; negative on error. */
+static int enc_speech_codec(struct msgb *msg,
+			    const struct gsm0808_speech_codec *sc)
 {
 	/* See also 3GPP TS 48.008 3.2.2.103 Speech Codec List */
 	uint8_t header = 0;
 	uint8_t *old_tail;
-	bool type_extended = false;
+	bool type_extended;
 
 	/* Note: Extended codec types are codec types that require 8 instead
 	 * of 4 bit to fully specify the selected codec. In the following,
 	 * we check if we work with an extended type or not. We also check
 	 * if the codec type is valid at all. */
-	switch(sc->type) {
+	switch (sc->type) {
 	case GSM0808_SCT_FR1:
 	case GSM0808_SCT_FR2:
 	case GSM0808_SCT_FR3:
@@ -235,8 +240,7 @@ static uint8_t enc_speech_codec(struct msgb *msg,
 		break;
 	default:
 		/* Invalid codec type specified */
-		OSMO_ASSERT(false);
-		break;
+		return -EINVAL;
 	}
 
 	old_tail = msg->tail;
@@ -255,7 +259,6 @@ static uint8_t enc_speech_codec(struct msgb *msg,
 		msgb_put_u8(msg, header);
 		msgb_put_u8(msg, sc->type);
 	} else {
-		OSMO_ASSERT(sc->type < 0x0f);
 		header |= sc->type;
 		msgb_put_u8(msg, header);
 	}
@@ -273,11 +276,13 @@ static uint8_t enc_speech_codec(struct msgb *msg,
 	case GSM0808_SCT_FR5:
 	case GSM0808_SCT_HR4:
 	case GSM0808_SCT_CSD:
-		OSMO_ASSERT((sc->cfg & 0xff00) == 0);
+		if (sc->cfg >> 8)
+			return -EINVAL;
 		msgb_put_u8(msg, (uint8_t) sc->cfg & 0xff);
 		break;
 	default:
-		OSMO_ASSERT(sc->cfg == 0);
+		if (sc->cfg != 0)
+			return -EINVAL;
 		break;
 	}
 
@@ -287,27 +292,38 @@ static uint8_t enc_speech_codec(struct msgb *msg,
 /*! Encode TS 08.08 Speech Codec IE
  *  \param[out] msg Message Buffer to which IE will be appended
  *  \param[in] sc Speech Codec to be encoded into IE
- *  \returns number of bytes appended to \a msg */
-uint8_t gsm0808_enc_speech_codec(struct msgb *msg,
-				 const struct gsm0808_speech_codec *sc)
+ *  \returns number of bytes appended to \a msg; negative on error */
+int gsm0808_enc_speech_codec2(struct msgb *msg,
+			      const struct gsm0808_speech_codec *sc)
 {
 	/*! See also 3GPP TS 48.008 3.2.2.103 Speech Codec List */
-	uint8_t *old_tail;
 	uint8_t *tlv_len;
-
-	OSMO_ASSERT(msg);
-	OSMO_ASSERT(sc);
+	int rc;
 
 	msgb_put_u8(msg, GSM0808_IE_SPEECH_CODEC);
 	tlv_len = msgb_put(msg, 1);
-	old_tail = msg->tail;
 
-	enc_speech_codec(msg, sc);
+	rc = enc_speech_codec(msg, sc);
+	if (rc < 0)
+		return rc;
 
-	*tlv_len = (uint8_t) (msg->tail - old_tail);
+	*tlv_len = rc;
 	return *tlv_len + 2;
 }
 
+/*! Deprecated: gsm0808_enc_speech_codec2() wrapper for backwards compatibility.
+ * Mimics the old behavior: crash on missing and/or invalid input. */
+uint8_t gsm0808_enc_speech_codec(struct msgb *msg,
+				 const struct gsm0808_speech_codec *sc)
+{
+	int rc;
+
+	rc = gsm0808_enc_speech_codec2(msg, sc);
+	OSMO_ASSERT(rc > 0);
+
+	return rc;
+}
+
 /*! Decode TS 08.08 Speech Codec IE
  *  \param[out] sc Caller-allocated memory for Speech Codec
  *  \param[in] elem IE value to be decoded
@@ -320,7 +336,6 @@ int gsm0808_dec_speech_codec(struct gsm0808_speech_codec *sc,
 	uint8_t header;
 	const uint8_t *old_elem = elem;
 
-	OSMO_ASSERT(sc);
 	if (!elem)
 		return -EINVAL;
 	if (len == 0)
@@ -392,35 +407,46 @@ int gsm0808_dec_speech_codec(struct gsm0808_speech_codec *sc,
 /*! Encode TS 08.08 Speech Codec list
  *  \param[out] msg  Message Buffer to which IE is to be appended
  *  \param[in] scl Speech Codec List to be encoded into IE
- *  \returns number of bytes added to \a msg */
-uint8_t gsm0808_enc_speech_codec_list(struct msgb *msg,
-				      const struct gsm0808_speech_codec_list *scl)
+ *  \returns number of bytes added to \a msg; negative on error */
+int gsm0808_enc_speech_codec_list2(struct msgb *msg,
+				   const struct gsm0808_speech_codec_list *scl)
 {
 	/*! See also 3GPP TS 48.008 3.2.2.103 Speech Codec List */
 	uint8_t *old_tail;
 	uint8_t *tlv_len;
 	unsigned int i;
-	uint8_t rc;
 	unsigned int bytes_used = 0;
 
-	OSMO_ASSERT(msg);
-	OSMO_ASSERT(scl);
-
 	msgb_put_u8(msg, GSM0808_IE_SPEECH_CODEC_LIST);
 	tlv_len = msgb_put(msg, 1);
 	old_tail = msg->tail;
 
 	for (i = 0; i < scl->len; i++) {
-		rc = enc_speech_codec(msg, &scl->codec[i]);
-		OSMO_ASSERT(rc >= 1);
+		int rc = enc_speech_codec(msg, &scl->codec[i]);
+		if (rc < 1)
+			return rc;
 		bytes_used += rc;
-		OSMO_ASSERT(bytes_used <= 255);
+		if (bytes_used > 0xff)
+			return -EOVERFLOW;
 	}
 
 	*tlv_len = (uint8_t) (msg->tail - old_tail);
 	return *tlv_len + 2;
 }
 
+/*! Deprecated: gsm0808_enc_speech_codec_list2() wrapper for backwards compatibility.
+ * Mimics the old behavior: crash on missing and/or invalid input. */
+uint8_t gsm0808_enc_speech_codec_list(struct msgb *msg,
+				      const struct gsm0808_speech_codec_list *scl)
+{
+	int rc;
+
+	rc = gsm0808_enc_speech_codec_list2(msg, scl);
+	OSMO_ASSERT(rc > 0);
+
+	return rc;
+}
+
 /*! Decode TS 08.08 Speech Codec list IE
  *  \param[out] scl Caller-provided memory to store codec list
  *  \param[in] elem IE value to be decoded
@@ -435,7 +461,6 @@ int gsm0808_dec_speech_codec_list(struct gsm0808_speech_codec_list *scl,
 	int rc;
 	uint8_t decoded = 0;
 
-	OSMO_ASSERT(scl);
 	if (!elem)
 		return -EINVAL;
 
@@ -472,8 +497,6 @@ uint8_t gsm0808_enc_channel_type(struct msgb *msg,
 	uint8_t *old_tail;
 	uint8_t *tlv_len;
 
-	OSMO_ASSERT(msg);
-	OSMO_ASSERT(ct);
 	OSMO_ASSERT(ct->perm_spch_len <= CHANNEL_TYPE_ELEMENT_MAXLEN - 2);
 
 	/* FIXME: Implement encoding support for Data
@@ -514,7 +537,6 @@ int gsm0808_dec_channel_type(struct gsm0808_channel_type *ct,
 	uint8_t byte;
 	const uint8_t *old_elem = elem;
 
-	OSMO_ASSERT(ct);
 	if (!elem)
 		return -EINVAL;
 	if (len < 3 || len > 11)
@@ -703,8 +725,6 @@ uint8_t gsm0808_enc_encrypt_info(struct msgb *msg,
 	uint8_t *old_tail;
 	uint8_t *tlv_len;
 
-	OSMO_ASSERT(msg);
-	OSMO_ASSERT(ei);
 	OSMO_ASSERT(ei->key_len <= ARRAY_SIZE(ei->key));
 	OSMO_ASSERT(ei->perm_algo_len <= ENCRY_INFO_PERM_ALGO_MAXLEN);
 
@@ -743,7 +763,6 @@ int gsm0808_dec_encrypt_info(struct gsm0808_encrypt_info *ei,
 	unsigned int perm_algo_len = 0;
 	const uint8_t *old_elem = elem;
 
-	OSMO_ASSERT(ei);
 	if (!elem)
 		return -EINVAL;
 	if (len == 0)
@@ -1003,9 +1022,6 @@ uint8_t gsm0808_enc_cell_id_list2(struct msgb *msg,
 	unsigned int i;
 	uint8_t id_discr;
 
-	OSMO_ASSERT(msg);
-	OSMO_ASSERT(cil);
-
 	msgb_put_u8(msg, GSM0808_IE_CELL_IDENTIFIER_LIST);
 	tlv_len = msgb_put(msg, 1);
 	old_tail = msg->tail;
@@ -1047,9 +1063,6 @@ uint8_t gsm0808_enc_cell_id_list(struct msgb *msg,
 	uint8_t *tlv_len;
 	unsigned int i;
 
-	OSMO_ASSERT(msg);
-	OSMO_ASSERT(cil);
-
 	msgb_put_u8(msg, GSM0808_IE_CELL_IDENTIFIER_LIST);
 	tlv_len = msgb_put(msg, 1);
 	old_tail = msg->tail;
@@ -1225,7 +1238,6 @@ int gsm0808_dec_cell_id_list2(struct gsm0808_cell_id_list2 *cil,
 	size_t bytes_elem = 0;
 	int list_len = 0;
 
-	OSMO_ASSERT(cil);
 	if (!elem)
 		return -EINVAL;
 	if (len == 0)
@@ -1293,7 +1305,6 @@ int gsm0808_dec_cell_id_list(struct gsm0808_cell_id_list *cil,
 	const uint8_t *old_elem = elem;
 	unsigned int item_count = 0;
 
-	OSMO_ASSERT(cil);
 	if (!elem)
 		return -EINVAL;
 	if (len == 0)
@@ -1451,9 +1462,6 @@ uint8_t gsm0808_enc_cell_id(struct msgb *msg, const struct gsm0808_cell_id *ci)
 		.id_list_len = 1,
 	};
 
-	OSMO_ASSERT(msg);
-	OSMO_ASSERT(ci);
-
 	ie_tag = msg->tail;
 	rc = gsm0808_enc_cell_id_list2(msg, &cil);
 
