@@ -19,10 +19,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 #include <stdint.h>
@@ -423,6 +419,29 @@ const char *gsm48_rr_msg_name(uint8_t msgtype)
 	return get_value_string(rr_msg_names, msgtype);
 }
 
+/* 3GPP TS 44.018 Table 10.4.2 */
+static const struct value_string rr_msg_type_short_names[] = {
+	{ GSM48_MT_RR_SH_SI10,		"System Information Type 10" },
+	{ GSM48_MT_RR_SH_FACCH,		"Notification/FACCH" },
+	{ GSM48_MT_RR_SH_UL_FREE,	"Uplink Free" },
+	{ GSM48_MT_RR_SH_MEAS_REP,	"Enhanced Measurement Report (uplink)" },
+	{ GSM48_MT_RR_SH_MEAS_INFO,	"Measurement Information (downlink)" },
+	{ GSM48_MT_RR_SH_VGCS_RECON,	"VBS/VGCS Reconfigure" },
+	{ GSM48_MT_RR_SH_VGCS_RECON2,	"VBS/VGCS Reconfigure2" },
+	{ GSM48_MT_RR_SH_VGCS_INFO,	"VGCS Additional Information" },
+	{ GSM48_MT_RR_SH_VGCS_SMS,	"VGCS SMS Information" },
+	{ GSM48_MT_RR_SH_SI10bis,	"System Information Type 10bis" },
+	{ GSM48_MT_RR_SH_SI10ter,	"System Information Type 10ter" },
+	{ GSM48_MT_RR_SH_VGCS_NEIGH,	"VGCS Neighbour Cell Information" },
+	{ GSM48_MT_RR_SH_APP_DATA,	"Notify Application Data" },
+	{ 0,				NULL }
+};
+
+/*! return string representation of RR Message Type using the RR short protocol discriminator */
+const char *gsm48_rr_short_pd_msg_name(uint8_t msgtype)
+{
+	return get_value_string(rr_msg_type_short_names, msgtype);
+}
 
 const struct value_string gsm48_chan_mode_names[] = {
 	{ GSM48_CMODE_SIGN,		"SIGNALLING" },
@@ -1365,7 +1384,7 @@ bool gsm48_ra_equal(const struct gprs_ra_id *raid1, const struct gprs_ra_id *rai
  *  Uses From Table 10.5.33 of GSM 04.08 to determine the number of
  *  paging sub-channels in the given control channel configuration
  */
-int gsm48_number_of_paging_subchannels(struct gsm48_control_channel_descr *chan_desc)
+int gsm48_number_of_paging_subchannels(const struct gsm48_control_channel_descr *chan_desc)
 {
 	unsigned int n_pag_blocks = gsm0502_get_n_pag_blocks(chan_desc);
 
