@@ -19,10 +19,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 
@@ -207,7 +203,24 @@ int gsm48_decode_bearer_cap(struct gsm_mncc_bearer_cap *bcap,
 	case GSM_MNCC_BCAP_SPEECH:
 		i = 1;
 		s = 0;
-		while(!(lv[i] & 0x80)) {
+		if ((lv[1] & 0x80) != 0) { /* octet 3a is absent */
+			switch (bcap->radio) {
+			case GSM48_BCAP_RRQ_FR_ONLY:
+				bcap->speech_ver[s++] = GSM48_BCAP_SV_FR;
+				break;
+			case GSM48_BCAP_RRQ_DUAL_HR:
+				bcap->speech_ver[s++] = GSM48_BCAP_SV_HR;
+				bcap->speech_ver[s++] = GSM48_BCAP_SV_FR;
+				break;
+			case GSM48_BCAP_RRQ_DUAL_FR:
+				bcap->speech_ver[s++] = GSM48_BCAP_SV_FR;
+				bcap->speech_ver[s++] = GSM48_BCAP_SV_HR;
+				break;
+			}
+			bcap->speech_ver[s] = -1; /* end of list */
+			return 0;
+		}
+		while (!(lv[i] & 0x80)) {
 			i++; /* octet 3a etc */
 			if (in_len < i)
 				return 0;
@@ -222,7 +235,7 @@ int gsm48_decode_bearer_cap(struct gsm_mncc_bearer_cap *bcap,
 	case GSM_MNCC_BCAP_UNR_DIG:
 	case GSM_MNCC_BCAP_FAX_G3:
 		i = 1;
-		while(!(lv[i] & 0x80)) {
+		while (!(lv[i] & 0x80)) {
 			i++; /* octet 3a etc */
 			if (in_len < i)
 				return 0;
@@ -236,7 +249,7 @@ int gsm48_decode_bearer_cap(struct gsm_mncc_bearer_cap *bcap,
 			return 0;
 		bcap->data.rate_adaption = (lv[i] >> 3) & 3;
 		bcap->data.sig_access = lv[i] & 7;
-		while(!(lv[i] & 0x80)) {
+		while (!(lv[i] & 0x80)) {
 			i++; /* octet 5a etc */
 			if (in_len < i)
 				return 0;
@@ -867,8 +880,9 @@ static int32_t smod(int32_t n, int32_t m)
  *  \param[in] cd Cell Channel Description IE
  *  \param[in] len Length of \a cd in bytes
  *  \returns 0 on success; negative on error */
-int gsm48_decode_freq_list(struct gsm_sysinfo_freq *f, uint8_t *cd,
-			   uint8_t len, uint8_t mask, uint8_t frqt)
+int gsm48_decode_freq_list(struct gsm_sysinfo_freq *f,
+			   const uint8_t *cd, uint8_t len,
+			   uint8_t mask, uint8_t frqt)
 {
 	int i;
 
