@@ -12,10 +12,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
@@ -69,6 +65,18 @@ static const struct gsm_mncc_bearer_cap bcap_speech_all = {
 	},
 };
 
+static const uint8_t speech_no3a_lv[] = { 0x01, 0xa0 };
+
+static const struct gsm_mncc_bearer_cap bcap_speech_no3a = {
+	.transfer =	GSM48_BCAP_ITCAP_SPEECH,
+	.mode =		GSM48_BCAP_TMOD_CIRCUIT,
+	.coding =	GSM48_BCAP_CODING_GSM_STD,
+	.radio =	GSM48_BCAP_RRQ_FR_ONLY,
+	.speech_ver = {
+		0, -1,
+	},
+};
+
 
 struct bcap_test {
 	const uint8_t *lv;
@@ -79,15 +87,17 @@ struct bcap_test {
 static const struct bcap_test bcap_tests[] = {
 	{ csd_9600_v110_lv, &bcap_csd_9600_v110, "CSD 9600/V.110/transparent" },
 	{ speech_all_lv, &bcap_speech_all, "Speech, all codecs" },
+	{ speech_no3a_lv, &bcap_speech_no3a, "Speech, without octet 3a" },
 };
 
-static int test_bearer_cap()
+static int test_bearer_cap(void)
 {
 	struct gsm_mncc_bearer_cap bc;
 	int i, rc;
 
 	for (i = 0; i < ARRAY_SIZE(bcap_tests); i++) {
 		struct msgb *msg = msgb_alloc(100, "test");
+		bool pass = false;
 		int lv_len;
 
 		memset(&bc, 0, sizeof(bc));
@@ -97,7 +107,7 @@ static int test_bearer_cap()
 		if (rc < 0) {
 			fprintf(stderr, "Error decoding %s\n",
 				bcap_tests[i].name);
-			return rc;
+			goto verdict;
 		}
 		if (memcmp(&bc, bcap_tests[i].bc, sizeof(bc))) {
 			fprintf(stderr, "Incorrect decoded result of %s:\n",
@@ -106,7 +116,7 @@ static int test_bearer_cap()
 				osmo_hexdump((uint8_t *) bcap_tests[i].bc, sizeof(bc)));
 			fprintf(stderr, " is:     %s\n",
 				osmo_hexdump((uint8_t *) &bc, sizeof(bc)));
-			return -1;
+			goto verdict;
 		}
 
 		/* also test re-encode? */
@@ -114,7 +124,7 @@ static int test_bearer_cap()
 		if (rc < 0) {
 			fprintf(stderr, "Error encoding %s\n",
 				bcap_tests[i].name);
-			return rc;
+			goto verdict;
 		}
 		lv_len = bcap_tests[i].lv[0]+1;
 		if (memcmp(msg->data, bcap_tests[i].lv, lv_len)) {
@@ -124,10 +134,14 @@ static int test_bearer_cap()
 				osmo_hexdump(bcap_tests[i].lv, lv_len));
 			fprintf(stderr, " is:     %s\n",
 				osmo_hexdump(msg->data, msg->len));
-			return -1;
+			goto verdict;
 		}
 
-		printf("Test `%s' passed\n", bcap_tests[i].name);
+		/* all checks passed */
+		pass = true;
+
+verdict:
+		printf("Test `%s' %sed\n", bcap_tests[i].name, pass ? "pass" : "fail");
 		msgb_free(msg);
 	}
 
@@ -1140,7 +1154,7 @@ struct mobile_identity_tc mobile_identity_tests[] = {
 	},
 };
 
-void test_struct_mobile_identity()
+void test_struct_mobile_identity(void)
 {
 	struct mobile_identity_tc *t;
 	printf("%s()\n", __func__);
@@ -1154,7 +1168,7 @@ void test_struct_mobile_identity()
 		rc = osmo_mobile_identity_decode_from_l3(&mi, msg, false);
 		msgb_free(msg);
 
-		printf("%s: rc = %d", t->label, rc);
+		printf("%s: %s", t->label, rc ? "rc != 0" : "rc == 0");
 		if (!rc) {
 			printf(", mi = %s", osmo_mobile_identity_to_str_c(OTC_SELECT, &mi));
 		}
@@ -1163,7 +1177,7 @@ void test_struct_mobile_identity()
 		    && ((rc != 0) || !osmo_mobile_identity_cmp(&mi, &t->expect_mi))) {
 			printf(" ok");
 		} else {
-			printf("  ERROR: Expected rc = %d", t->expect_rc);
+			printf("  ERROR: Got rc = %d, expected rc = %d", rc, t->expect_rc);
 			if (!t->expect_rc)
 				printf(", mi = %s", osmo_mobile_identity_to_str_c(OTC_SELECT, &t->expect_mi));
 		}
@@ -1302,7 +1316,7 @@ static const struct bcd_number_test {
 	},
 };
 
-static void test_bcd_number_encode_decode()
+static void test_bcd_number_encode_decode(void)
 {
 	const struct bcd_number_test *test;
 	uint8_t buf_enc[0xff] = { 0xff };
@@ -1536,7 +1550,7 @@ static void test_random_range_encoding(int range, int max_arfcn_num)
 	}
 }
 
-static void test_range_encoding()
+static void test_range_encoding(void)
 {
 	int *arfcns;
 	int arfcns_num = 0;
@@ -1597,7 +1611,7 @@ static int range512[] = {
 			__FILE__, __LINE__, (int) res, # cmp, (int) wanted);	\
 	}
 
-static void test_arfcn_filter()
+static void test_arfcn_filter(void)
 {
 	int arfcns[50], i, res, f0_included;
 	for (i = 0; i < ARRAY_SIZE(arfcns); ++i)
@@ -1630,7 +1644,7 @@ static void test_arfcn_filter()
 		VERIFY(arfcns[i], ==, ((i + 1) * 2) - 1);
 }
 
-static void test_print_encoding()
+static void test_print_encoding(void)
 {
 	int rc;
 	int w[17];
@@ -1655,7 +1669,7 @@ static void test_print_encoding()
 	printf("Range512: %s\n", osmo_hexdump(chan_list, ARRAY_SIZE(chan_list)));
 }
 
-static void test_si_range_helpers()
+static void test_si_range_helpers(void)
 {
 	int ws[(sizeof(freqs1)/sizeof(freqs1[0]))];
 	int i, f0 = 0xFFFFFF;
@@ -1694,7 +1708,7 @@ static void test_si_range_helpers()
 	VERIFY(f0, ==, 1);
 }
 
-static void test_power_ctrl()
+static void test_power_ctrl(void)
 {
 	int8_t rc8;
 	int rc;
@@ -1736,6 +1750,37 @@ static void test_power_ctrl()
 	VERIFY(rc, <, 0);
 }
 
+static void test_rach_tx_integer_raw2val(void)
+{
+	unsigned int raw;
+	for (raw = 0; raw <= 0x0f; raw++) {
+		unsigned int val = rach_tx_integer_raw2val(raw);
+		printf("rach_tx_integer_raw2val(0x0%x): %u slots used to spread transmission\n",
+			raw, val);
+	}
+}
+
+static void test_gsm_gsmtime2fn(void)
+{
+	struct gsm_time gsm_time;
+	uint32_t fn;
+	uint32_t fn_recovered;
+
+	for (fn = 0; fn < 42432; fn++) {
+		gsm_time.t1 = (fn / 1326) % 32;
+		gsm_time.t2 = fn % 26;
+		gsm_time.t3 = fn % 51;
+
+		fn_recovered = gsm_gsmtime2fn(&gsm_time);
+
+		if (fn_recovered != fn) {
+			printf(" Wrong frame number computed! t1=%d, t2=%d, t3=%d ==> fn=%d, expected fn=%d\n",
+			       gsm_time.t1, gsm_time.t2, gsm_time.t3, fn_recovered, fn);
+			OSMO_ASSERT(false);
+		}
+	}
+}
+
 int main(int argc, char **argv)
 {
 	test_bearer_cap();
@@ -1754,6 +1799,8 @@ int main(int argc, char **argv)
 	test_print_encoding();
 	test_range_encoding();
 	test_power_ctrl();
+	test_rach_tx_integer_raw2val();
+	test_gsm_gsmtime2fn();
 
 	return EXIT_SUCCESS;
 }
