@@ -14,10 +14,6 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
- *
  */
 
 #include <osmocom/gsm/ipa.h>
@@ -346,7 +342,7 @@ static struct {
 	{ "DeafBeddedBabeAcceededFadedDecaff", 32, 32, false, false },
 };
 
-bool test_is_hexstr()
+bool test_is_hexstr(void)
 {
 	int i;
 	bool pass = true;
@@ -770,6 +766,65 @@ static void isqrt_test(void)
 	}
 }
 
+static void mod_test_mod(int x, int y, int expected_result)
+{
+	int result;
+	result = x % y;
+	printf(" %d mod %d = %d = %d\n", x, y, result, expected_result);
+	OSMO_ASSERT(result == expected_result);
+}
+
+static void mod_test_mod_flr(int x, int y, int expected_result)
+{
+	int result;
+	result = OSMO_MOD_FLR(x, y);
+	printf(" %d mod_flr %d = %d = %d\n", x, y, result, expected_result);
+	OSMO_ASSERT(result == expected_result);
+}
+
+static void mod_test_mod_euc(int x, int y, int expected_result)
+{
+	int result;
+	result = OSMO_MOD_EUC(x, y);
+	printf(" %d mod_euc %d = %d = %d\n", x, y, result, expected_result);
+	OSMO_ASSERT(result == expected_result);
+}
+
+static void mod_test(void)
+{
+	/* See also: Daan Leijen, Division and Modulus for Computer
+	 * Scientists, section 1.3 */
+
+	printf("\nTesting built in truncated modulo for comparison:\n");
+	mod_test_mod(8, 3, 2);
+	mod_test_mod(8, -3, 2);
+	mod_test_mod(-8, 3, -2);
+	mod_test_mod(-8, -3, -2);
+	mod_test_mod(1, 2, 1);
+	mod_test_mod(1, -2, 1);
+	mod_test_mod(-1, 2, -1);
+	mod_test_mod(-1, -2, -1);
+
+	printf("\nTesting OSMO_MOD_FLR():\n");
+	mod_test_mod_flr(8, 3, 2);
+	mod_test_mod_flr(8, -3, -1);
+	mod_test_mod_flr(-8, 3, 1);
+	mod_test_mod_flr(-8, -3, -2);
+	mod_test_mod_flr(1, 2, 1);
+	mod_test_mod_flr(1, -2, -1);
+	mod_test_mod_flr(-1, 2, 1);
+	mod_test_mod_flr(-1, -2, -1);
+
+	printf("\nTesting OSMO_MOD_EUC():\n");
+	mod_test_mod_euc(8, 3, 2);
+	mod_test_mod_euc(8, -3, 2);
+	mod_test_mod_euc(-8, 3, 1);
+	mod_test_mod_euc(-8, -3, 1);
+	mod_test_mod_euc(1, 2, 1);
+	mod_test_mod_euc(1, -2, 1);
+	mod_test_mod_euc(-1, 2, 1);
+	mod_test_mod_euc(-1, -2, 1);
+}
 
 struct osmo_sockaddr_to_str_and_uint_test_case {
 	uint16_t port;
@@ -1027,7 +1082,7 @@ struct osmo_str_tolowupper_test_data osmo_str_tolowupper_tests[] = {
 };
 
 
-static void osmo_str_tolowupper_test()
+static void osmo_str_tolowupper_test(void)
 {
 	int i;
 	char buf[128];
@@ -1201,7 +1256,7 @@ int strbuf_cascade(char *buf, size_t buflen)
 	return sb.chars_needed;
 }
 
-void strbuf_test()
+void strbuf_test(void)
 {
 	char buf[256];
 	int rc;
@@ -1234,7 +1289,7 @@ void strbuf_test()
 	printf("(need %d chars, had size=63) %s\n", rc, buf);
 }
 
-void strbuf_test_nolen()
+void strbuf_test_nolen(void)
 {
 	char buf[20];
 	struct osmo_strbuf sb = { .buf = buf, .len = sizeof(buf) };
@@ -1260,7 +1315,7 @@ static void startswith_test_str(const char *str, const char *startswith_str, boo
 		printf("   ERROR: EXPECTED %s\n", expect_rc ? "true" : "false");
 }
 
-static void startswith_test()
+static void startswith_test(void)
 {
 	printf("\n%s()\n", __func__);
 	startswith_test_str(NULL, NULL, true);
@@ -1305,7 +1360,7 @@ static char *foo_name_c_zero_null(void *ctx, const char *arg)
         OSMO_NAME_C_IMPL(ctx, 0, NULL, foo_name_buf, arg)
 }
 
-static void name_c_impl_test()
+static void name_c_impl_test(void)
 {
 	char *test_strs[] = {
 		"test",
@@ -1694,13 +1749,20 @@ struct float_str_to_int_test float_str_to_int_tests[] = {
 };
 const char *errno_str(int rc)
 {
-	if (rc == -EINVAL)
+	switch (rc) {
+	case -EINVAL:
 		return "=-EINVAL";
-	if (rc == -ERANGE)
+	case -ERANGE:
 		return "=-ERANGE";
-	return "";
+	case -E2BIG:
+		return "=-E2BIG";
+	case -EOVERFLOW:
+		return "=-EOVERFLOW";
+	default:
+		return "";
+	}
 }
-void test_float_str_to_int()
+void test_float_str_to_int(void)
 {
 	const struct float_str_to_int_test *t;
 	printf("--- %s\n", __func__);
@@ -1862,7 +1924,7 @@ struct int_to_float_str_test int_to_float_str_tests[] = {
 	{ 23, -9223372036854775807, "-0.00009223372036854775807" },
 	{ 23, INT64_MIN, "-ERR" },
 };
-void test_int_to_float_str()
+void test_int_to_float_str(void)
 {
 	const struct int_to_float_str_test *t;
 	printf("--- %s\n", __func__);
@@ -1884,6 +1946,190 @@ void test_int_to_float_str()
 	}
 }
 
+struct str_to_int_test {
+	const char *str;
+	int base;
+	int min_val;
+	int max_val;
+	int expect_rc;
+	int expect_val;
+};
+/* Avoid using INT_MAX and INT_MIN because that would produce different test output on different architectures */
+struct str_to_int_test str_to_int_tests[] = {
+	{ NULL, 10, -1000, 1000, -EINVAL, 0 },
+	{ "", 10, -1000, 1000, -EINVAL, 0 },
+	{ " ", 10, -1000, 1000, -EINVAL, 0 },
+	{ "-", 10, -1000, 1000, -EINVAL, 0 },
+	{ "--", 10, -1000, 1000, -EINVAL, 0 },
+	{ "+", 10, -1000, 1000, -EINVAL, 0 },
+	{ "++", 10, -1000, 1000, -EINVAL, 0 },
+
+	{ "0", 10, -1000, 1000, 0, 0 },
+	{ "1", 10, -1000, 1000, 0, 1 },
+	{ "+1", 10, -1000, 1000, 0, 1 },
+	{ "-1", 10, -1000, 1000, 0, -1 },
+	{ "1000", 10, -1000, 1000, 0, 1000 },
+	{ "+1000", 10, -1000, 1000, 0, 1000 },
+	{ "-1000", 10, -1000, 1000, 0, -1000 },
+	{ "1001", 10, -1000, 1000, -ERANGE, 1001 },
+	{ "+1001", 10, -1000, 1000, -ERANGE, 1001 },
+	{ "-1001", 10, -1000, 1000, -ERANGE, -1001 },
+
+	{ "0", 16, -1000, 1000, 0, 0 },
+	{ "1", 16, -1000, 1000, 0, 1 },
+	{ "0x1", 16, -1000, 1000, 0, 1 },
+	{ "+1", 16, -1000, 1000, 0, 1 },
+	{ "-1", 16, -1000, 1000, 0, -1 },
+	{ "+0x1", 16, -1000, 1000, 0, 1 },
+	{ "-0x1", 16, -1000, 1000, 0, -1 },
+	{ "3e8", 16, -1000, 1000, 0, 1000 },
+	{ "3E8", 16, -1000, 1000, 0, 1000 },
+	{ "0x3e8", 16, -1000, 1000, 0, 1000 },
+	{ "0x3E8", 16, -1000, 1000, 0, 1000 },
+	{ "+3e8", 16, -1000, 1000, 0, 1000 },
+	{ "+3E8", 16, -1000, 1000, 0, 1000 },
+	{ "+0x3e8", 16, -1000, 1000, 0, 1000 },
+	{ "+0x3E8", 16, -1000, 1000, 0, 1000 },
+	{ "-3e8", 16, -1000, 1000, 0, -1000 },
+	{ "-3E8", 16, -1000, 1000, 0, -1000 },
+	{ "-0x3e8", 16, -1000, 1000, 0, -1000 },
+	{ "-0x3E8", 16, -1000, 1000, 0, -1000 },
+	{ "3e9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "3E9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "0x3e9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "0x3E9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "+3e9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "+3E9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "+0x3e9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "+0x3E9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "-3e9", 16, -1000, 1000, -ERANGE, -1001 },
+	{ "-3E9", 16, -1000, 1000, -ERANGE, -1001 },
+	{ "-0x3e9", 16, -1000, 1000, -ERANGE, -1001 },
+	{ "-0x3E9", 16, -1000, 1000, -ERANGE, -1001 },
+
+	{ "garble", 10, -1000, 1000, -EINVAL, 0 },
+	{ "-garble", 10, -1000, 1000, -EINVAL, 0 },
+	{ "0x123", 10, -1000, 1000, -E2BIG, 0 },
+	{ "123potatoes", 10, -1000, 1000, -E2BIG, 123 },
+	{ "123 potatoes", 10, -1000, 1000, -E2BIG, 123 },
+	{ "123 ", 10, -1000, 1000, -E2BIG, 123 },
+	{ "123.4", 10, -1000, 1000, -E2BIG, 123 },
+};
+void test_str_to_int(void)
+{
+	const struct str_to_int_test *t;
+	printf("--- %s\n", __func__);
+	for (t = str_to_int_tests; (t - str_to_int_tests) < ARRAY_SIZE(str_to_int_tests); t++) {
+		int rc;
+		int val;
+		rc = osmo_str_to_int(&val, t->str, t->base, t->min_val, t->max_val);
+		printf("osmo_str_to_int(%s, %d, %d, %d) -> rc=%d%s val=%d\n",
+		       osmo_quote_str(t->str, -1), t->base, t->min_val, t->max_val, rc, errno_str(rc), val);
+
+		if (rc != t->expect_rc)
+			printf("  ERROR: expected rc=%d%s\n", t->expect_rc, errno_str(t->expect_rc));
+		if (val != t->expect_val)
+			printf("  ERROR: expected val=%d\n", t->expect_val);
+	}
+}
+
+struct str_to_int64_test {
+	const char *str;
+	int base;
+	int64_t min_val;
+	int64_t max_val;
+	int expect_rc;
+	int64_t expect_val;
+};
+struct str_to_int64_test str_to_int64_tests[] = {
+	{ NULL, 10, -1000, 1000, -EINVAL, 0 },
+	{ "", 10, -1000, 1000, -EINVAL, 0 },
+	{ " ", 10, -1000, 1000, -EINVAL, 0 },
+	{ "-", 10, -1000, 1000, -EINVAL, 0 },
+	{ "--", 10, -1000, 1000, -EINVAL, 0 },
+	{ "+", 10, -1000, 1000, -EINVAL, 0 },
+	{ "++", 10, -1000, 1000, -EINVAL, 0 },
+
+	{ "0", 10, -1000, 1000, 0, 0 },
+	{ "1", 10, -1000, 1000, 0, 1 },
+	{ "+1", 10, -1000, 1000, 0, 1 },
+	{ "-1", 10, -1000, 1000, 0, -1 },
+	{ "1000", 10, -1000, 1000, 0, 1000 },
+	{ "+1000", 10, -1000, 1000, 0, 1000 },
+	{ "-1000", 10, -1000, 1000, 0, -1000 },
+	{ "1001", 10, -1000, 1000, -ERANGE, 1001 },
+	{ "+1001", 10, -1000, 1000, -ERANGE, 1001 },
+	{ "-1001", 10, -1000, 1000, -ERANGE, -1001 },
+
+	{ "0", 16, -1000, 1000, 0, 0 },
+	{ "1", 16, -1000, 1000, 0, 1 },
+	{ "0x1", 16, -1000, 1000, 0, 1 },
+	{ "+1", 16, -1000, 1000, 0, 1 },
+	{ "-1", 16, -1000, 1000, 0, -1 },
+	{ "+0x1", 16, -1000, 1000, 0, 1 },
+	{ "-0x1", 16, -1000, 1000, 0, -1 },
+	{ "3e8", 16, -1000, 1000, 0, 1000 },
+	{ "3E8", 16, -1000, 1000, 0, 1000 },
+	{ "0x3e8", 16, -1000, 1000, 0, 1000 },
+	{ "0x3E8", 16, -1000, 1000, 0, 1000 },
+	{ "+3e8", 16, -1000, 1000, 0, 1000 },
+	{ "+3E8", 16, -1000, 1000, 0, 1000 },
+	{ "+0x3e8", 16, -1000, 1000, 0, 1000 },
+	{ "+0x3E8", 16, -1000, 1000, 0, 1000 },
+	{ "-3e8", 16, -1000, 1000, 0, -1000 },
+	{ "-3E8", 16, -1000, 1000, 0, -1000 },
+	{ "-0x3e8", 16, -1000, 1000, 0, -1000 },
+	{ "-0x3E8", 16, -1000, 1000, 0, -1000 },
+	{ "3e9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "3E9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "0x3e9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "0x3E9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "+3e9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "+3E9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "+0x3e9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "+0x3E9", 16, -1000, 1000, -ERANGE, 1001 },
+	{ "-3e9", 16, -1000, 1000, -ERANGE, -1001 },
+	{ "-3E9", 16, -1000, 1000, -ERANGE, -1001 },
+	{ "-0x3e9", 16, -1000, 1000, -ERANGE, -1001 },
+	{ "-0x3E9", 16, -1000, 1000, -ERANGE, -1001 },
+
+	{ "garble", 10, -1000, 1000, -EINVAL, 0 },
+	{ "-garble", 10, -1000, 1000, -EINVAL, 0 },
+	{ "0x123", 10, -1000, 1000, -E2BIG, 0 },
+	{ "123potatoes", 10, -1000, 1000, -E2BIG, 123 },
+	{ "123 potatoes", 10, -1000, 1000, -E2BIG, 123 },
+	{ "123 ", 10, -1000, 1000, -E2BIG, 123 },
+	{ "123.4", 10, -1000, 1000, -E2BIG, 123 },
+
+	{ "-9223372036854775808", 10, INT64_MIN, INT64_MAX, 0, INT64_MIN },
+	{ "9223372036854775807", 10, INT64_MIN, INT64_MAX, 0, INT64_MAX },
+
+	{ "-9223372036854775809", 10, INT64_MIN, INT64_MAX, -EOVERFLOW, INT64_MIN },
+	{ "9223372036854775808", 10, INT64_MIN, INT64_MAX, -EOVERFLOW, INT64_MAX },
+
+	{ "-9223372036854775808", 10, -1000, 1000, -ERANGE, INT64_MIN },
+	{ "9223372036854775807", 10, -1000, 1000, -ERANGE, INT64_MAX },
+	{ "-9223372036854775809", 10, -1000, 1000, -EOVERFLOW, INT64_MIN },
+	{ "9223372036854775808", 10, -1000, 1000, -EOVERFLOW, INT64_MAX },
+};
+void test_str_to_int64(void)
+{
+	const struct str_to_int64_test *t;
+	printf("--- %s\n", __func__);
+	for (t = str_to_int64_tests; (t - str_to_int64_tests) < ARRAY_SIZE(str_to_int64_tests); t++) {
+		int rc;
+		int64_t val;
+		rc = osmo_str_to_int64(&val, t->str, t->base, t->min_val, t->max_val);
+		printf("osmo_str_to_int64(%s, %d, %"PRId64", %"PRId64") -> rc=%d%s val=%"PRId64"\n",
+		       osmo_quote_str(t->str, -1), t->base, t->min_val, t->max_val, rc, errno_str(rc), val);
+
+		if (rc != t->expect_rc)
+			printf("  ERROR: expected rc=%d%s\n", t->expect_rc, errno_str(t->expect_rc));
+		if (val != t->expect_val)
+			printf("  ERROR: expected val=%"PRId64"\n", t->expect_val);
+	}
+}
+
 int main(int argc, char **argv)
 {
 	static const struct log_info log_info = {};
@@ -1901,6 +2147,7 @@ int main(int argc, char **argv)
 	str_escape3_test();
 	str_quote3_test();
 	isqrt_test();
+	mod_test();
 	osmo_sockaddr_to_str_and_uint_test();
 	osmo_str_tolowupper_test();
 	strbuf_test();
@@ -1911,5 +2158,7 @@ int main(int argc, char **argv)
 	osmo_strnchr_test();
 	test_float_str_to_int();
 	test_int_to_float_str();
+	test_str_to_int();
+	test_str_to_int64();
 	return 0;
 }
